import re
import os
import sys

mydir = os.path.dirname( __file__ )
lmpdir = os.path.join(mydir, '..', 'lmpset')
sys.path.append(lmpdir)
from infchain import infchain

def chain(file, bondtype, degree, loc='./'):
    """
    The method to create a single molecule for a finite chain,  specialised for polymers.
    chain(file, bondtype, degree, loc='./')
    file: Your molecule system name on your .data file.
    degree: Degree of polymerisation.
    bondtype: The bond type between the start and the end of the polymer. 
    loc: File Location. The default is your current location.
    Example:
        Input:
            from MCPoly.lmpset import chain
            chain('BioPolymer2', 11, 16)
        Output in Poly1_16x.data:
            1314 atoms
            1345 bonds
            2468 angles
            3382 dihedrals
            753 impropers
            
            9 atom types
            12 bond types
            21 angle types
            31 dihedral types
            3 improper types
            
            ...
            82 1 83 1
            
            ...
    """
    opath = os.getcwd()
    os.chdir(loc)
    k = 0
    w1 = 0
    w2 = 0
    w3 = 0
    w4 = 0
    w5 = 0
    n1 = 0
    t2 = 0
    t3 = 0
    t4 = 0
    t5 = 0
    H = []
    lx = []
    lost2_u = []
    lost2_d = []
    lost3_u = []
    lost3_d = []
    lost4_u = []
    lost4_d = []
    lost5_u = []
    lost5_d = []
    Hloc = [[0.0, 0.0, 0.0], [0.0, 0.0, 0.0]]
    l = []
    real = [0, 0, 0]
    inp = open('{0}.data'.format(file), 'r')
    for line in inp:
        a1 = re.search(r' atoms', line)
        a2 = re.search(r' bonds', line)
        a3 = re.search(r' angles', line)
        a4 = re.search(r' dihedrals', line)
        a5 = re.search(r' impropers', line)
        a6 = re.search(r' xlo xhi', line)
        a7 = re.search(r' ylo yhi', line)
        a8 = re.search(r' zlo zhi', line)
        m1 = re.search(r'Masses', line)
        if a1:
            b1 = re.search(r'[0-9]+', line)
            tipnum = eval(b1.group(0))
        elif a2:
            b2 = re.search(r'[0-9]+', line)
            tipnum2 = eval(b2.group(0))
        elif a3:
            b3 = re.search(r'[0-9]+', line)
            tipnum3 = eval(b3.group(0))
        elif a4:
            b4 = re.search(r'[0-9]+', line)
            tipnum4 = eval(b4.group(0))
        elif a5:
            b5 = re.search(r'[0-9]+', line)
            tipnum5 = eval(b5.group(0))
        elif a6:
            b6 = re.findall(r'\-?[0-9]+\.?[0-9]*', line)
            tipgapx = eval(b6[1]) - eval(b6[0])
            xlo = eval(b6[0])
            xhi = eval(b6[1])
        elif a7:
            b7 = re.findall(r'\-?[0-9]+\.?[0-9]*', line)
            gapy = eval(b7[1]) - eval(b7[0])
            ylo = eval(b7[0])
        elif a8:
            b8 = re.findall(r'\-?[0-9]+\.?[0-9]*', line)
            gapz = eval(b8[1]) - eval(b8[0])
            zlo = eval(b8[0])
        if n1 == 1:
            l1 = re.findall(r'\-?[0-9]+\.?[0-9]*', line)
            if l1 == []:
                continue
            else:
                n1 = 2
        if n1 == 2:
            l1 = re.findall(r'\-?[0-9]+\.?[0-9]*', line)
            if l1 == []:
                n1 = 0
                continue
            if eval(l1[-1]) < 1.1:
                H.append(eval(l1[0]))
        if m1:
            n1 = 1
        c1 = re.search('Atoms', line)
        c2 = re.search('Bonds', line)
        c3 = re.search('Angles', line)
        c4 = re.search('Dihedrals', line)
        c5 = re.search('Impropers', line)
        if c1:
            w1 = 1
        elif c2:
            w2 = 1
        elif c3:
            w3 = 1
        elif c4:
            w4 = 1
        elif c5:
            w5 = 1
        elif w1 == 1:
            l1 = re.findall(r'\-?[0-9]+\.?[0-9]*', line)
            if len(l1) >= 6:
                l.append(l1)
                if eval(l1[0]) == tipnum:
                    w1 = 2
        elif w1 == 2:
            xmin = 99999
            xmax = -99999
            xmaxnum = 0
            xminnum = 0
            realxmin = 99999
            realxmax = -99999
            realxmaxnum = 0
            realxminnum = 0
            for i4 in range(len(l)):
                if eval(l[i4][2]) not in H:
                    continue
                if eval(l[i4][-3]) > xmax:
                    xmax = eval(l[i4][-3])
                    xmaxnum = eval(l[i4][0])
                    Hloc[1] = [eval(l[i4][-3]), eval(l[i4][-2]), eval(l[i4][-1])]
                if eval(l[i4][-3]) < xmin:
                    xmin = eval(l[i4][-3])
                    xminnum = eval(l[i4][0])
                    Hloc[0] = [eval(l[i4][-3]), eval(l[i4][-2]), eval(l[i4][-1])]
            for i4 in range(len(l)):
                if eval(l[i4][2]) in H:
                    continue
                if eval(l[i4][-3]) > realxmax:
                    realxmax = eval(l[i4][-3])
                    realxmaxnum = eval(l[i4][0])
                if eval(l[i4][-3]) < realxmin:
                    realxmin = eval(l[i4][-3])
                    realxminnum = eval(l[i4][0])
            w1 = 0
        elif w2 == 1:
            l1 = re.findall(r'[0-9]+', line)
            if len(l1) == 4:
                l.append(l1)
                if eval(l1[0]) == tipnum2:
                    w2 = 2
                    j = 1
        elif w2 == 2:
            for i4 in range(len(l)):
                if eval(l[i4][2]) in [xminnum, xmaxnum]:
                    if eval(l[i4][2]) == xminnum:
                        lost2_d = [eval(l[i4][-2]), eval(l[i4][-1])]
                    elif eval(l[i4][2]) == xmaxnum:
                        lost2_u = [eval(l[i4][-2]), eval(l[i4][-1])]
                    continue
                elif eval(l[i4][3]) in [xminnum, xmaxnum]:
                    if eval(l[i4][3]) == xminnum:
                        lost2_d = [eval(l[i4][-2]), eval(l[i4][-1])]
                    elif eval(l[i4][3]) == xmaxnum:
                        lost2_u = [eval(l[i4][-2]), eval(l[i4][-1])]
                    continue
                j = j + 1
            k = k + 1
            l = []
            t2 = j
            j = 1
            w2 = 0
            k = 0
        elif w3 == 1:
            l1 = re.findall(r'[0-9]+', line)
            if len(l1) == 5:
                l.append(l1)
                if eval(l1[0]) == tipnum3:
                    w3 = 2
                    j = 1
        elif w3 == 2:
            for i4 in range(len(l)):
                if eval(l[i4][2]) in [xminnum, xmaxnum]:
                    if eval(l[i4][2]) == xminnum:
                        lost3_d.append([eval(l[i4][-3]), eval(l[i4][-2]), eval(l[i4][-1])])
                    elif eval(l[i4][2]) == xmaxnum:
                        lost3_u.append([eval(l[i4][-3]), eval(l[i4][-2]), eval(l[i4][-1])])
                    continue
                elif eval(l[i4][3]) in [xminnum, xmaxnum]:
                    if eval(l[i4][3]) == xminnum:
                        lost3_d.append([eval(l[i4][-3]), eval(l[i4][-2]), eval(l[i4][-1])])
                    if eval(l[i4][3]) == xmaxnum:
                        lost3_u.append([eval(l[i4][-3]), eval(l[i4][-2]), eval(l[i4][-1])])
                    continue
                elif eval(l[i4][4]) in [xminnum, xmaxnum]:
                    if eval(l[i4][4]) == xminnum:
                        lost3_d.append([eval(l[i4][-3]), eval(l[i4][-2]), eval(l[i4][-1])])
                    if eval(l[i4][4]) == xmaxnum:
                        lost3_u.append([eval(l[i4][-3]), eval(l[i4][-2]), eval(l[i4][-1])])
                    continue
                j = j + 1
            k = k + 1
            l = []
            t3 = j
            j = 1
            k = 0
            w3 = 0
        elif w4 == 1:
            l1 = re.findall(r'[0-9]+', line)
            if len(l1) == 6:
                l.append(l1)
                if eval(l1[0]) == tipnum4:
                    w4 = 2
                    j = 1
        elif w4 == 2:
            for i4 in range(len(l)):
                if eval(l[i4][2]) in [xminnum, xmaxnum]:
                    if eval(l[i4][2]) == xminnum:
                        lost4_d.append([eval(l[i4][-4]), eval(l[i4][-3]),\
                                        eval(l[i4][-2]), eval(l[i4][-1])])
                    elif eval(l[i4][2]) == xmaxnum:
                        lost4_u.append([eval(l[i4][-4]), eval(l[i4][-3]),\
                                        eval(l[i4][-2]), eval(l[i4][-1])])
                    continue
                elif eval(l[i4][3]) in [xminnum, xmaxnum]:
                    if eval(l[i4][3]) == xminnum:
                        lost4_d.append([eval(l[i4][-4]), eval(l[i4][-3]),\
                                        eval(l[i4][-2]), eval(l[i4][-1])])
                    elif eval(l[i4][3]) == xmaxnum:
                        lost4_u.append([eval(l[i4][-4]), eval(l[i4][-3]),\
                                        eval(l[i4][-2]), eval(l[i4][-1])])
                    continue
                elif eval(l[i4][4]) in [xminnum, xmaxnum]:
                    if eval(l[i4][4]) == xminnum:
                        lost4_d.append([eval(l[i4][-4]), eval(l[i4][-3]),\
                                        eval(l[i4][-2]), eval(l[i4][-1])])
                    elif eval(l[i4][4]) == xmaxnum:
                        lost4_u.append([eval(l[i4][-4]), eval(l[i4][-3]),\
                                        eval(l[i4][-2]), eval(l[i4][-1])])
                    continue
                elif eval(l[i4][5]) in [xminnum, xmaxnum]:
                    if eval(l[i4][5]) == xminnum:
                        lost4_d.append([eval(l[i4][-4]), eval(l[i4][-3]),\
                                        eval(l[i4][-2]), eval(l[i4][-1])])
                    elif eval(l[i4][5]) == xmaxnum:
                        lost4_u.append([eval(l[i4][-4]), eval(l[i4][-3]),\
                                        eval(l[i4][-2]), eval(l[i4][-1])])
                    continue
                j = j + 1
            k = k + 1
            l = []
            t4 = j
            j = 1
            k = 0
            w4 = 0
        elif w5 == 1:
            l1 = re.findall(r'[0-9]+', line)
            if len(l1) == 6:
                l.append(l1)
                if eval(l1[0]) == tipnum5:
                    w5 = 2
                    j = 1
        elif w5 == 2:
            for i4 in range(len(l)):
                if eval(l[i4][2]) in [xminnum, xmaxnum]:
                    if eval(l[i4][2]) == xminnum:
                        lost5_d.append([eval(l[i4][-4]), eval(l[i4][-3]),\
                                        eval(l[i4][-2]), eval(l[i4][-1])])
                    elif eval(l[i4][2]) == xmaxnum:
                        lost5_u.append([eval(l[i4][-4]), eval(l[i4][-3]),\
                                        eval(l[i4][-2]), eval(l[i4][-1])])
                    continue
                elif eval(l[i4][3]) in [xminnum, xmaxnum]:
                    if eval(l[i4][3]) == xminnum:
                        lost5_d.append([eval(l[i4][-4]), eval(l[i4][-3]),\
                                        eval(l[i4][-2]), eval(l[i4][-1])])
                    elif eval(l[i4][3]) == xmaxnum:
                        lost5_u.append([eval(l[i4][-4]), eval(l[i4][-3]),\
                                        eval(l[i4][-2]), eval(l[i4][-1])])
                    continue
                elif eval(l[i4][4]) in [xminnum, xmaxnum]:
                    if eval(l[i4][4]) == xminnum:
                        lost5_d.append([eval(l[i4][-4]), eval(l[i4][-3]),\
                                        eval(l[i4][-2]), eval(l[i4][-1])])
                    elif eval(l[i4][4]) == xmaxnum:
                        lost5_u.append([eval(l[i4][-4]), eval(l[i4][-3]),\
                                        eval(l[i4][-2]), eval(l[i4][-1])])
                    continue
                elif eval(l[i4][5]) in [xminnum, xmaxnum]:
                    if eval(l[i4][5]) == xminnum:
                        lost5_d.append([eval(l[i4][-4]), eval(l[i4][-3]),\
                                        eval(l[i4][-2]), eval(l[i4][-1])])
                    elif eval(l[i4][5]) == xmaxnum:
                        lost5_u.append([eval(l[i4][-4]), eval(l[i4][-3]),\
                                        eval(l[i4][-2]), eval(l[i4][-1])])
                    continue
                j = j + 1
            k = k + 1
            l = []
            t5 = j
            j = 1
            k = 0
            w4 = 0
    k = 0
    w1 = 0
    w2 = 0
    w3 = 0
    w4 = 0
    w5 = 0
    gap = 0
    l = []
    try:
        med = open('{0}_Chain.data'.format(file), 'r')
    except:
        infchain(file, bondtype, loc)
        med = open('{0}_Chain.data'.format(file), 'r')
    for line in med:
        a1 = re.search(r' atoms', line)
        a2 = re.search(r' bonds', line)
        a3 = re.search(r' angles', line)
        a4 = re.search(r' dihedrals', line)
        a5 = re.search(r' impropers', line)
        a6 = re.search(r' xlo xhi', line)
        
        m1 = re.search(r'Masses', line)
        if a1:
            b1 = re.search(r'[0-9]+', line)
            num = eval(b1.group(0))
        elif a2:
            b2 = re.search(r'[0-9]+', line)
            num2 = eval(b2.group(0))
        elif a3:
            b3 = re.search(r'[0-9]+', line)
            num3 = eval(b3.group(0))
        elif a4:
            b4 = re.search(r'[0-9]+', line)
            num4 = eval(b4.group(0))
        elif a5:
            b5 = re.search(r'[0-9]+', line)
            num5 = eval(b5.group(0))
        elif a6:
            b6 = re.findall(r'\-?[0-9]+\.?[0-9]*', line)
            gapx = eval(b6[1]) - eval(b6[0])
        if n1 == 1:
            l1 = re.findall(r'\-?[0-9]+\.?[0-9]*', line)
            if l1 == []:
                continue
            else:
                n1 = 2
        if n1 == 2:
            l1 = re.findall(r'\-?[0-9]+\.?[0-9]*', line)
            if l1 == []:
                n1 = 0
                continue
            if eval(l1[-1])<1.1:
                H.append(eval(l1[0]))
        if m1:
            n1 = 1
        c1 = re.search('Atoms', line)
        c2 = re.search('Bonds', line)
        c3 = re.search('Angles', line)
        c4 = re.search('Dihedrals', line)
        c5 = re.search('Impropers', line)
        if c1:
            w1 = 1
        elif c2:
            w2 = 1
        elif c3:
            w3 = 1
        elif c4:
            w4 = 1
        elif c5:
            w5 = 1
        elif w1 == 1:
            l1 = re.findall(r'\-?[0-9]+\.?[0-9]*', line)
            if len(l1) >= 6:
                l.append(l1)
                if eval(l1[0]) == tipnum:
                    w1 = 2
        elif w1 == 2:
            w1 = 0
        elif w2 == 1:
            l1 = re.findall(r'[0-9]+', line)
            if len(l1) == 4:
                l.append(l1)
                if eval(l1[0]) == tipnum2:
                    w2 = 2
                    j = 1
        elif w2 == 2:
            for i4 in range(len(l)):
                if eval(l[i4][2]) in [xminnum, xmaxnum]:
                    continue
                if eval(l[i4][3]) in [xminnum, xmaxnum]:
                    continue
                j = j + 1
            k = k + 1
            l = []
            t2 = j
            j = 1
            w2 = 0
            k = 0
        elif w3 == 1:
            l1 = re.findall(r'[0-9]+', line)
            if len(l1) == 5:
                l.append(l1)
                if eval(l1[0]) == tipnum3:
                    w3 = 2
                    j = 1
        elif w3 == 2:
            for i4 in range(len(l)):
                if eval(l[i4][2]) in [xminnum, xmaxnum]:
                    continue
                if eval(l[i4][3]) in [xminnum, xmaxnum]:
                    continue
                if eval(l[i4][4]) in [xminnum, xmaxnum]:
                    continue
                j = j + 1
            k = k + 1
            l = []
            t3 = j
            j = 1
            k = 0
            w3 = 0
        elif w4 == 1:
            l1 = re.findall(r'[0-9]+', line)
            if len(l1) == 6:
                l.append(l1)
                if eval(l1[0]) == tipnum4:
                    w4 = 2
                    j = 1
        elif w4 == 2:
            for i4 in range(len(l)):
                if eval(l[i4][2]) in [xminnum, xmaxnum]:
                    continue
                if eval(l[i4][3]) in [xminnum, xmaxnum]:
                    continue
                if eval(l[i4][4]) in [xminnum, xmaxnum]:
                    continue
                if eval(l[i4][5]) in [xminnum, xmaxnum]:
                    continue
                j = j + 1
            k = k + 1
            l = []
            t4 = j
            j = 1
            k = 0
            w4 = 0
        elif w5 == 1:
            l1 = re.findall(r'[0-9]+', line)
            if len(l1) == 6:
                l.append(l1)
                if eval(l1[0]) == tipnum5:
                    w5 = 2
                    j = 1
        elif w5 == 2:
            for i4 in range(len(l)):
                if eval(l[i4][2]) in [xminnum, xmaxnum]:
                    continue
                if eval(l[i4][3]) in [xminnum, xmaxnum]:
                    continue
                if eval(l[i4][4]) in [xminnum, xmaxnum]:
                    continue
                if eval(l[i4][5]) in [xminnum, xmaxnum]:
                    continue
                j = j + 1
            k = k + 1
            l = []
            t5 = j
            j = 1
            k = 0
            w4 = 0
    try:
        f = open('{0}_{1}x.data'.format(file, degree), 'x')
    except:
        f = open('{0}_{1}x.data'.format(file, degree), 'w')
    inp.close()
    k = 0
    w1 = 0
    w2 = 0
    w3 = 0
    w4 = 0
    w5 = 0
    gap = 0
    l = []
    all_x = []
    inp = open('{0}.data'.format(file), 'r')
    for line in inp:
        a1 = re.search(r' atoms', line)
        a2 = re.search(r' bonds', line)
        a3 = re.search(r' angles', line)
        a4 = re.search(r' dihedrals', line)
        a5 = re.search(r' impropers', line)
        a6 = re.search(r' xlo xhi', line)
        a7 = re.search(r' ylo yhi', line)
        a8 = re.search(r' zlo zhi', line)
        c1 = re.search('Atoms', line)
        c2 = re.search('Bonds', line)
        c3 = re.search('Angles', line)
        c4 = re.search('Dihedrals', line)
        c5 = re.search('Impropers', line)
        if a1:
            b1 = re.search(r'[0-9]+', line)
            f.write('{0} atoms\n'.format((tipnum-2)*degree + 2))
            #allnum = b1.group(0)
        elif a2:
            b2 = re.search(r'[0-9]+', line)
            f.write('{0} bonds\n'.format(tipnum2*degree - (degree-1)))
            #allnum2 = b2.group(0)
        elif a3:
            f.write('{0} angles\n'.format(tipnum3 + num3*(degree-1)))
            #allnum3 = b3.group(0)
        elif a4:
            b4 = re.search(r'[0-9]+', line)
            f.write('{0} dihedrals\n'.format(tipnum4 + num4*(degree-1)))
            #allnum4 = b4.group(0)
        elif a5:
            b5 = re.search(r'[0-9]+', line)
            f.write('{0} impropers\n'.format(tipnum5 + num5*(degree-1)))
            #allnum5 = b5.group(0)
        elif a6:
            x1 = re.findall(r'\-?[0-9]+\.?[0-9]*', line)
            xd = eval(x1[1]) - eval(x1[0])
            f.write('{0:.5f} {1:.5f} xlo xhi\n'.format(0.0,\
                                                       tipgapx*(degree-1) + gapx))
        elif a7:
            y1 = re.findall(r'\-?[0-9]+\.?[0-9]*', line)
            yd = eval(y1[1]) - eval(y1[0])
            f.write('{0:.5f} {1:.5f} ylo yhi\n'.format(0.0, gapy))
        elif a8:
            z1 = re.findall(r'\-?[0-9]+\.?[0-9]*', line)
            zd = eval(z1[1]) - eval(z1[0])
            f.write('{0:.5f} {1:.5f} zlo zhi\n'.format(0.0, gapz))
        elif c1:
            w1 = 1
            f.write(line)
            f.write('\n')
        elif c2:
            w2 = 1
            f.write(line)
            f.write('\n')
        elif c3:
            w3 = 1
            f.write(line)
            f.write('\n')
        elif c4:
            w4 = 1
            f.write(line)
            f.write('\n')
        elif c5:
            w5 = 1
            f.write(line)
            f.write('\n')
        elif w1 == 1:
            l1 = re.findall(r'\-?[0-9]+\.?[0-9]*', line)
            if len(l1) >= 6:
                l.append(l1)
                if eval(l1[0]) == tipnum:
                    w1 = 2
                    j = 1
        elif w1 == 2:
            distance = 0
            allnum = 0
            for i3 in range(degree):
                for i4 in range(eval(b1.group(0))):
                    if i3 != 0 and i3 != degree-1:
                        if i4 + 1 in [xminnum, xmaxnum]:
                            gap = gap + 1
                            j = j + 1
                            continue
                    elif i3 == 0:
                        if i4 + 1 == xmaxnum:
                            gap = gap + 1
                            j = j + 1
                            all_x.append(eval(l[i4][-3]))
                            continue
                    elif i3 == degree - 1:
                        if i4 + 1 == xminnum:
                            gap = gap + 1
                            j = j + 1
                            continue
                    #print(i3, j-gap, eval(l[i4][-3]), eval(l[i4][-2]), eval(l[i4][-1]))
                    #if realxmin< = realxmax:
                    f.write('{0} {1} '.format(j-gap, eval(l[i4][1])))
                    f.write('{0} {1:.4f} '.format(eval(l[i4][2]), eval(l[i4][3])))
                    f.write('{0:.5f} '.format(eval(l[i4][-3]) + distance - xlo))
                    f.write('{0:.5f} '.format(eval(l[i4][-2]) - ylo))
                    f.write('{0:.5f}\n'.format(eval(l[i4][-1]) - zlo))
#                    else:
#                        f.write('{0} {1} {2} {3:.4f} {4:.5f} {5:.5f} {6:.5f}\n'.format(j-gap,\
#eval(l[i4][1]), eval(l[i4][2]), eval(l[i4][3]),
#eval(l[i4][-3]) + distance-xhi, 
#eval(l[i4][-2])-ylo, eval(l[i4][-1])-zlo))
                    j = j + 1
                    if i3 == 0:
                        all_x.append(eval(l[i4][-3]))
                if i3 != 0 and i3 != degree - 1:
                    distance = distance + tipgapx
                    allnum = allnum + num - 2
                else:
                    distance = distance + gapx
                    allnum = allnum + num - 1
            l = []
            j = 1
            w1 = 0
            f.write('\n')
        elif w2 == 1:
            l1 = re.findall(r'[0-9]+', line)
            if len(l1) == 4:
                l.append(l1)
                if eval(l1[0]) == tipnum2:
                    lx = l
                    w2 = 2
                    j = 1
        elif w2 == 2:
            every = 0
            for i3 in range(degree):
                repeated = []
                #print(i3)
                for i4 in range(eval(b2.group(0))):
                    if i3 != 0 and i3 != degree - 1:
                        if eval(l[i4][2]) in [xminnum, xmaxnum]:
                            #print(l[i4])
                            continue
                        elif eval(l[i4][2])>min([xminnum, xmaxnum]):
                            l[i4][2] = str(eval(l[i4][2]) - 1)
                        elif eval(l[i4][2])>max([xminnum, xmaxnum]):
                            l[i4][2] = str(eval(l[i4][2]) - 2)
                        if eval(l[i4][3]) in [xminnum, xmaxnum]:
                            if eval(l[i4][2]) >= min([xminnum, xmaxnum]):
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            elif eval(l[i4][2]) + 1 >= max([xminnum, xmaxnum]):
                                l[i4][2] = str(eval(l[i4][2]) + 2)
                            continue
                        elif eval(l[i4][3])>min([xminnum, xmaxnum]):
                            l[i4][3] = str(eval(l[i4][3]) - 1)
                        elif eval(l[i4][3])>max([xminnum, xmaxnum]):
                            l[i4][3] = str(eval(l[i4][3]) - 2)
                    elif i3 == 0:
                        if eval(l[i4][2]) == xmaxnum:
                            continue
                        elif eval(l[i4][2]) > xmaxnum:
                            l[i4][2] = str(eval(l[i4][2]) - 1)
                        if eval(l[i4][3]) == xmaxnum:
                            if eval(l[i4][2]) >= xmaxnum:
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            continue
                        elif eval(l[i4][3]) > xmaxnum:
                            l[i4][3] = str(eval(l[i4][3]) - 1)
                    elif i3 == degree-1:
                        if eval(l[i4][2]) == xminnum:
                            #print('{0} {1}\n'.format(eval(l[i4][2]), eval(l[i4][3])))
                            continue
                        elif eval(l[i4][2]) > xminnum:
                            l[i4][2] = str(eval(l[i4][2]) - 1)
                        if eval(l[i4][3]) == xminnum:
                            if eval(l[i4][2]) >= xminnum:
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            continue
                        elif eval(l[i4][3]) > xminnum:
                            l[i4][3] = str(eval(l[i4][3]) - 1)
                    f.write('{0} {1} {2} {3}\n'.format(j, eval(l[i4][1]),\
                                                       eval(l[i4][2]) + every, eval(l[i4][3]) + every))
                    #print('{0} {1} {2} {3}'.format(j, eval(l[i4][1]), eval(l[i4][2]) + every, eval(l[i4][3]) + every))
                    j = j + 1
                if i3 != 0:
                    if realxminnum <= realxmaxnum:
                        f.write('{0} {1} {2} {3}\n'.format(j, bondtype, realxminnum + every,\
                                                           realxmaxnum + every - num - 1))
                    else:
                        if i3 == degree - 1:
                            f.write('{0} {1} {2} {3}\n'.format(j, bondtype, realxminnum + every,\
                                                               realxmaxnum + every - num - 1))
                        else:
                            f.write('{0} {1} {2} {3}\n'.format(j, bondtype, realxminnum + every - 1,\
                                                               realxmaxnum + every - num - 1))
                        #if all_x[eval(l[i4][2])-1]< = all_x[eval(l[i4][3])-1]:
                        #    if eval(l[i4][2])>eval(l[i4][3]) + tipnum*i3:
                        #        f.write('{0} {1} {2} {3}\n'.format(j, bondtype,\
                        #                                           realxminnum + every, realxmaxnum + every - num))
                        #    else:
                        #        f.write('{0} {1} {2} {3}\n'.format(j, bondtype,\
                        #                                           realxminnum + every-num, realxmaxnum + every))
                        #elif all_x[eval(l[i4][2])-1]>all_x[eval(l[i4][3])-1]:
                        #    if eval(l[i4][2])>eval(l[i4][3]) + tipnum*i3:
                        #        f.write('{0} {1} {2} {3}\n'.format(j, bondtype,\
                        #                                           realxminnum + every-num, realxmaxnum + every))
                        #    else:
                        #        f.write('{0} {1} {2} {3}\n'.format(j, bondtype,\
                        #                                           realxminnum + every, realxmaxnum + every - num))
                    #print('{0} {1} {2} {3}'.format(j, bondtype,\
                    #                               realxminnum + every-1, realxmaxnum + every - num - 1))
                    j = j + 1
                #l = lx
                if i3 == 0 or i3 == degree - 1:
                    every = every + tipnum - 1
                else:
                    every = every + tipnum - 2
                #print(l[-1], xmaxnum)
                for i4 in range(eval(b2.group(0))):
                    if i3 != 0 and i3 != degree - 1:
                        if (([eval(l[i4][2]), eval(l[i4][3])] == lost2_u
                                 or [eval(l[i4][2]), eval(l[i4][3])] == lost2_d) and
                            [eval(l[i4][2]), eval(l[i4][3])] not in repeated):
                            repeated.append([eval(l[i4][2]), eval(l[i4][3])])
                            continue
                        elif eval(l[i4][2]) >= min([xminnum, xmaxnum]):
                            l[i4][2] = str(eval(l[i4][2]) + 1)
                        elif eval(l[i4][2]) + 1 >= max([xminnum, xmaxnum]):
                            l[i4][2] = str(eval(l[i4][2]) + 2)
                        if eval(l[i4][3]) >= min([xminnum, xmaxnum]):
                            l[i4][3] = str(eval(l[i4][3]) + 1)
                        elif eval(l[i4][3]) + 1 >= max([xminnum, xmaxnum]):
                            l[i4][3] = str(eval(l[i4][3]) + 2)
                    elif i3 == 0:
                        if (([eval(l[i4][2]), eval(l[i4][3])] == lost2_u
                                 or [eval(l[i4][2]), eval(l[i4][3])] == lost2_d) and
                            [eval(l[i4][2]), eval(l[i4][3])] not in repeated):
                            repeated.append([eval(l[i4][2]), eval(l[i4][3])])
                            continue
                        if eval(l[i4][2]) >= xmaxnum:
                            l[i4][2] = str(eval(l[i4][2]) + 1)  
                        if eval(l[i4][3]) >= xmaxnum:
                            l[i4][3] = str(eval(l[i4][3]) + 1)
                    elif i3 == degree-1:
                        if (([eval(l[i4][2]), eval(l[i4][3])] == lost2_u
                                 or [eval(l[i4][2]), eval(l[i4][3])] == lost2_d) and
                            [eval(l[i4][2]), eval(l[i4][3])] not in repeated):
                            repeated.append([eval(l[i4][2]), eval(l[i4][3])])
                            continue
                        if eval(l[i4][2]) >= xminnum:
                            l[i4][2] = str(eval(l[i4][2]) + 1)
                        if eval(l[i4][3]) >= xminnum:
                            l[i4][3] = str(eval(l[i4][3]) + 1)
                #if i3 == 1:
                    #print(l)
            k = k + 1
            l = []
            j = 1
            w2 = 0
            k = 0
            f.write('\n')
        elif w3 == 1:
            l1 = re.findall(r'[0-9]+', line)
            if len(l1) == 5:
                l.append(l1)
                if eval(l1[0]) == tipnum3:
                    w3 = 2
                    j = 1
            #lx = l
        elif w3 == 2:
            every = 0
            for i3 in range(degree):
                repeated = []
                for i4 in range(len(l)):
                    if i3 == 0:
                        if eval(l[i4][2]) == xmaxnum:
                            continue
                        elif eval(l[i4][2]) > xmaxnum:
                            l[i4][2] = str(eval(l[i4][2])-1) 
                        if eval(l[i4][3]) == xmaxnum:
                            if eval(l[i4][2]) >= xmaxnum:
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            continue
                        elif eval(l[i4][3]) > xmaxnum:
                            l[i4][3] = str(eval(l[i4][3])-1)
                        if eval(l[i4][4]) == xmaxnum:
                            if eval(l[i4][2]) >= xmaxnum:
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            if eval(l[i4][3]) >= xmaxnum:
                                l[i4][3] = str(eval(l[i4][3]) + 1)
                            continue
                        elif eval(l[i4][4]) > xmaxnum:
                            l[i4][4] = str(eval(l[i4][4])-1)  
                    elif i3 == degree-1:
                        if eval(l[i4][2]) == xminnum:
                            continue
                        elif eval(l[i4][2]) > xminnum:
                            l[i4][2] = str(eval(l[i4][2])-1) 
                        if eval(l[i4][3]) == xminnum:
                            if eval(l[i4][2]) >= xminnum:
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            continue
                        elif eval(l[i4][3]) > xmaxnum:
                            l[i4][3] = str(eval(l[i4][3])-1)
                        if eval(l[i4][4]) == xminnum:
                            if eval(l[i4][2]) >= xminnum:
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            if eval(l[i4][3]) >= xminnum:
                                l[i4][3] = str(eval(l[i4][3]) + 1)
                            continue
                        elif eval(l[i4][4]) > xminnum:
                            l[i4][4] = str(eval(l[i4][4])-1)           
                    else:
                        if eval(l[i4][2]) in [xminnum, xmaxnum]:
                            continue
                        elif eval(l[i4][2])>min([xminnum, xmaxnum]):
                            l[i4][2] = str(eval(l[i4][2])-1)
                        elif eval(l[i4][2])>max([xminnum, xmaxnum]):
                            l[i4][2] = str(eval(l[i4][2])-2)  
                        if eval(l[i4][3]) in [xminnum, xmaxnum]:
                            if eval(l[i4][2]) >= min([xminnum, xmaxnum]):
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            elif eval(l[i4][2]) + 1 >= max([xminnum, xmaxnum]):
                                l[i4][2] = str(eval(l[i4][2]) + 2)
                            continue
                        elif eval(l[i4][3])>min([xminnum, xmaxnum]):
                            l[i4][3] = str(eval(l[i4][3])-1)
                        elif eval(l[i4][3])>max([xminnum, xmaxnum]):
                            l[i4][3] = str(eval(l[i4][3])-2)  
                        if eval(l[i4][4]) in [xminnum, xmaxnum]:
                            if eval(l[i4][2]) >= min([xminnum, xmaxnum]):
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            elif eval(l[i4][2]) + 1 >= max([xminnum, xmaxnum]):
                                l[i4][2] = str(eval(l[i4][2]) + 2)
                            if eval(l[i4][3]) >= min([xminnum, xmaxnum]):
                                l[i4][3] = str(eval(l[i4][3]) + 1)
                            elif eval(l[i4][3]) + 1 >= max([xminnum, xmaxnum]):
                                l[i4][3] = str(eval(l[i4][3]) + 2)
                            continue
                        elif eval(l[i4][4])>min([xminnum, xmaxnum]):
                            l[i4][4] = str(eval(l[i4][4])-1)
                        elif eval(l[i4][4])>max([xminnum, xmaxnum]):
                            l[i4][4] = str(eval(l[i4][4])-2)  
                    f.write('{0} {1} {2} {3} {4}\n'.format(j, eval(l[i4][1]),\
                                                           eval(l[i4][2]) + every, eval(l[i4][3]) + every, eval(l[i4][4]) + every))
                    j = j + 1
                if i3 == 0 or i3 == degree-1:
                    every = every + tipnum-1
                else:
                    every = every + tipnum-2
                for i4 in range(len(l)):
                    if i3 == 0:
                        if (([eval(l[i4][2]), eval(l[i4][3]), eval(l[i4][4])] in lost3_u
                                 or [eval(l[i4][2]), eval(l[i4][3]), eval(l[i4][4])] in lost3_d) and
                            [eval(l[i4][2]), eval(l[i4][3]), eval(l[i4][4])] not in repeated):
                            repeated.append([eval(l[i4][2]), eval(l[i4][3]), eval(l[i4][4])])
                            continue
                        elif eval(l[i4][2]) >= xmaxnum:
                            l[i4][2] = str(eval(l[i4][2]) + 1) 
                        if eval(l[i4][3]) >= xmaxnum:
                            l[i4][3] = str(eval(l[i4][3]) + 1)
                        if eval(l[i4][4]) >= xmaxnum:
                            l[i4][4] = str(eval(l[i4][4]) + 1)  
                    elif i3 == degree-1:
                        if (([eval(l[i4][2]), eval(l[i4][3]), eval(l[i4][4])] in lost3_u
                                 or [eval(l[i4][2]), eval(l[i4][3]), eval(l[i4][4])] in lost3_d) and
                            [eval(l[i4][2]), eval(l[i4][3]), eval(l[i4][4])] not in repeated):
                            repeated.append([eval(l[i4][2]), eval(l[i4][3]), eval(l[i4][4])])
                            continue
                        elif eval(l[i4][2]) >= xminnum:
                            l[i4][2] = str(eval(l[i4][2]) + 1) 
                        if eval(l[i4][3]) >= xminnum:
                            l[i4][3] = str(eval(l[i4][3]) + 1)
                        if eval(l[i4][4]) >= xminnum:
                            l[i4][4] = str(eval(l[i4][4]) + 1)           
                    else:
                        if (([eval(l[i4][2]), eval(l[i4][3]), eval(l[i4][4])] in lost3_u
                                 or [eval(l[i4][2]), eval(l[i4][3]), eval(l[i4][4])] in lost3_d) and
                            [eval(l[i4][2]), eval(l[i4][3]), eval(l[i4][4])] not in repeated):
                            repeated.append([eval(l[i4][2]), eval(l[i4][3]), eval(l[i4][4])])
                            continue
                        if eval(l[i4][2]) >= min([xminnum, xmaxnum]):
                            l[i4][2] = str(eval(l[i4][2]) + 1)
                        elif eval(l[i4][2]) + 1 >= max([xminnum, xmaxnum]):
                            l[i4][2] = str(eval(l[i4][2]) + 2)  
                        if eval(l[i4][3]) >= min([xminnum, xmaxnum]):
                            l[i4][3] = str(eval(l[i4][3]) + 1)
                        elif eval(l[i4][3]) + 1 >= max([xminnum, xmaxnum]):
                            l[i4][3] = str(eval(l[i4][3]) + 2)  
                        if eval(l[i4][4]) >= min([xminnum, xmaxnum]):
                            l[i4][4] = str(eval(l[i4][4]) + 1)
                        elif eval(l[i4][4]) + 1 >= max([xminnum, xmaxnum]):
                            l[i4][4] = str(eval(l[i4][4]) + 2)
                #l = lx
            k = k + 1
            l = []
            j = 1
            k = 0
            w3 = 0
            f.write('\n')
        elif w4 == 1:
            l1 = re.findall(r'[0-9]+', line)
            if len(l1) == 6:
                l.append(l1)
                if eval(l1[0]) == tipnum4:
                    w4 = 2
                    j = 1
            #lx = l
        elif w4 == 2:
            every = 0
            for i3 in range(degree):
                repeated = []
                for i4 in range(len(l)):
                    if i3 == 0:
                        if eval(l[i4][2]) == xmaxnum:
                            continue
                        elif eval(l[i4][2]) > xmaxnum:
                            l[i4][2] = str(eval(l[i4][2])-1)
                        if eval(l[i4][3]) == xmaxnum:
                            if eval(l[i4][2]) >= xmaxnum:
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            continue
                        elif eval(l[i4][3]) > xmaxnum:
                            l[i4][3] = str(eval(l[i4][3])-1)  
                        if eval(l[i4][4]) == xmaxnum:
                            if eval(l[i4][2]) >= xmaxnum:
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            if eval(l[i4][3]) >= xmaxnum:
                                l[i4][3] = str(eval(l[i4][3]) + 1)
                            continue
                        elif eval(l[i4][4]) > xmaxnum:
                            l[i4][4] = str(eval(l[i4][4])-1)
                        if eval(l[i4][5]) == xmaxnum:
                            if eval(l[i4][2]) >= xmaxnum:
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            if eval(l[i4][3]) >= xmaxnum:
                                l[i4][3] = str(eval(l[i4][3]) + 1)
                            if eval(l[i4][4]) >= xmaxnum:
                                l[i4][4] = str(eval(l[i4][4]) + 1)
                            continue
                        elif eval(l[i4][5]) > xmaxnum:
                            l[i4][5] = str(eval(l[i4][5])-1)
                    elif i3 == degree-1:
                        if eval(l[i4][2]) == xminnum:
                            continue
                        elif eval(l[i4][2]) > xminnum:
                            l[i4][2] = str(eval(l[i4][2])-1)  
                        if eval(l[i4][3]) == xminnum:
                            if eval(l[i4][2]) >= xminnum:
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            continue
                        elif eval(l[i4][3]) > xminnum:
                            l[i4][3] = str(eval(l[i4][3])-1)  
                        if eval(l[i4][4]) == xminnum:
                            if eval(l[i4][2]) >= xminnum:
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            if eval(l[i4][3]) >= xminnum:
                                l[i4][3] = str(eval(l[i4][3]) + 1)
                            continue
                        elif eval(l[i4][4]) > xminnum:
                            l[i4][4] = str(eval(l[i4][4])-1)
                        if eval(l[i4][5]) == xminnum:
                            if eval(l[i4][2]) >= xminnum:
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            if eval(l[i4][3]) >= xminnum:
                                l[i4][3] = str(eval(l[i4][3]) + 1)
                            if eval(l[i4][4]) >= xminnum:
                                l[i4][4] = str(eval(l[i4][4]) + 1)
                            continue
                        elif eval(l[i4][5]) > xminnum:
                            l[i4][5] = str(eval(l[i4][5])-1)
                    else:
                        if eval(l[i4][2]) in [xminnum, xmaxnum]:
                            continue
                        elif eval(l[i4][2])>min([xminnum, xmaxnum]):
                            l[i4][2] = str(eval(l[i4][2])-1)
                        elif eval(l[i4][2])>max([xminnum, xmaxnum]):
                            l[i4][2] = str(eval(l[i4][2])-2)  
                        if eval(l[i4][3]) in [xminnum, xmaxnum]:
                            if eval(l[i4][2]) >= min([xminnum, xmaxnum]):
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            elif eval(l[i4][2]) + 1 >= max([xminnum, xmaxnum]):
                                l[i4][2] = str(eval(l[i4][2]) + 2)
                            continue
                        elif eval(l[i4][3])>min([xminnum, xmaxnum]):
                            l[i4][3] = str(eval(l[i4][3])-1)
                        elif eval(l[i4][3])>max([xminnum, xmaxnum]):
                            l[i4][3] = str(eval(l[i4][3])-2)  
                        if eval(l[i4][4]) in [xminnum, xmaxnum]:
                            if eval(l[i4][2]) >= min([xminnum, xmaxnum]):
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            elif eval(l[i4][2]) + 1 >= max([xminnum, xmaxnum]):
                                l[i4][2] = str(eval(l[i4][2]) + 2)
                            if eval(l[i4][3]) >= min([xminnum, xmaxnum]):
                                l[i4][3] = str(eval(l[i4][3]) + 1)
                            elif eval(l[i4][3]) + 1 >= max([xminnum, xmaxnum]):
                                l[i4][3] = str(eval(l[i4][3]) + 2)
                            continue
                        elif eval(l[i4][4])>min([xminnum, xmaxnum]):
                            l[i4][4] = str(eval(l[i4][4])-1)
                        elif eval(l[i4][4])>max([xminnum, xmaxnum]):
                            l[i4][4] = str(eval(l[i4][4])-2)  
                        if eval(l[i4][5]) in [xminnum, xmaxnum]:
                            if eval(l[i4][2]) >= min([xminnum, xmaxnum]):
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            elif eval(l[i4][2]) + 1 >= max([xminnum, xmaxnum]):
                                l[i4][2] = str(eval(l[i4][2]) + 2)
                            if eval(l[i4][3]) >= min([xminnum, xmaxnum]):
                                l[i4][3] = str(eval(l[i4][3]) + 1)
                            elif eval(l[i4][3]) + 1 >= max([xminnum, xmaxnum]):
                                l[i4][3] = str(eval(l[i4][3]) + 2)
                            if eval(l[i4][4]) >= min([xminnum, xmaxnum]):
                                l[i4][4] = str(eval(l[i4][4]) + 1)
                            elif eval(l[i4][4]) + 1 >= max([xminnum, xmaxnum]):
                                l[i4][4] = str(eval(l[i4][2]) + 2)
                            continue
                        elif eval(l[i4][5])>min([xminnum, xmaxnum]):
                            l[i4][5] = str(eval(l[i4][5])-1)
                        elif eval(l[i4][5])>max([xminnum, xmaxnum]):
                            l[i4][5] = str(eval(l[i4][5])-2)
                    f.write('{0} {1} {2} {3} {4} {5}\n'.format(j, eval(l[i4][1]),\
                                                               eval(l[i4][2]) + every, eval(l[i4][3]) + every,\
                                                               eval(l[i4][4]) + every, eval(l[i4][5]) + every))
                    j = j + 1
                for i4 in range(len(l)):
                    if i3 == 0:
                        if (([eval(l[i4][2]), eval(l[i4][3]),\
                              eval(l[i4][4]), eval(l[i4][5])] in lost4_u
                                 or [eval(l[i4][2]), eval(l[i4][3]),\
                                    eval(l[i4][4]), eval(l[i4][5])] in lost4_d) and
                            [eval(l[i4][2]), eval(l[i4][3]),\
                             eval(l[i4][4]), eval(l[i4][5])] not in repeated):
                            repeated.append([eval(l[i4][2]), eval(l[i4][3]),\
                                             eval(l[i4][4]), eval(l[i4][5])])
                            continue
                        if eval(l[i4][2]) >= xmaxnum:
                            l[i4][2] = str(eval(l[i4][2]) + 1)
                        if eval(l[i4][3]) >= xmaxnum:
                            l[i4][3] = str(eval(l[i4][3]) + 1)
                        if eval(l[i4][4]) >= xmaxnum:
                            l[i4][4] = str(eval(l[i4][4]) + 1)
                        if eval(l[i4][5]) >= xmaxnum:
                            l[i4][5] = str(eval(l[i4][5]) + 1)
                    elif i3 == degree - 1:
                        if (([eval(l[i4][2]), eval(l[i4][3]),\
                              eval(l[i4][4]), eval(l[i4][5])] in lost4_u
                                 or [eval(l[i4][2]), eval(l[i4][3]),\
                                     eval(l[i4][4]), eval(l[i4][5])] in lost4_d) and
                            [eval(l[i4][2]), eval(l[i4][3]),\
                             eval(l[i4][4]), eval(l[i4][5])] not in repeated):
                            repeated.append([eval(l[i4][2]), eval(l[i4][3]),\
                                             eval(l[i4][4]), eval(l[i4][5])])
                            continue
                        if eval(l[i4][2]) >= xminnum:
                            l[i4][2] = str(eval(l[i4][2]) + 1)
                        if eval(l[i4][3]) >= xminnum:
                            l[i4][3] = str(eval(l[i4][3]) + 1)
                        if eval(l[i4][4]) >= xminnum:
                            l[i4][4] = str(eval(l[i4][4]) + 1)
                        if eval(l[i4][5]) >= xminnum:
                            l[i4][5] = str(eval(l[i4][5]) + 1)
                    else:
                        if (([eval(l[i4][2]), eval(l[i4][3]),\
                             eval(l[i4][4]), eval(l[i4][5])] in lost4_u
                                or [eval(l[i4][2]), eval(l[i4][3]),\
                                    eval(l[i4][4]), eval(l[i4][5])] in lost4_d) and
                            [eval(l[i4][2]), eval(l[i4][3]),\
                             eval(l[i4][4]), eval(l[i4][5])] not in repeated):
                            repeated.append([eval(l[i4][2]), eval(l[i4][3]),\
                                             eval(l[i4][4]), eval(l[i4][5])])
                            continue
                        if eval(l[i4][2]) >= min([xminnum, xmaxnum]):
                            l[i4][2] = str(eval(l[i4][2]) + 1)
                        elif eval(l[i4][2]) + 1 >= max([xminnum, xmaxnum]):
                            l[i4][2] = str(eval(l[i4][2]) + 2)  
                        if eval(l[i4][3]) >= min([xminnum, xmaxnum]):
                            l[i4][3] = str(eval(l[i4][3]) + 1)
                        elif eval(l[i4][3]) + 1 >= max([xminnum, xmaxnum]):
                            l[i4][3] = str(eval(l[i4][3]) + 2)  
                        if eval(l[i4][4]) >= min([xminnum, xmaxnum]):
                            l[i4][4] = str(eval(l[i4][4]) + 1)
                        elif eval(l[i4][4]) + 1 >= max([xminnum, xmaxnum]):
                            l[i4][4] = str(eval(l[i4][4]) + 2)  
                        if eval(l[i4][5]) >= min([xminnum, xmaxnum]):
                            l[i4][5] = str(eval(l[i4][5]) + 1)
                        elif eval(l[i4][5]) + 1 >= max([xminnum, xmaxnum]):
                            l[i4][5] = str(eval(l[i4][5]) + 2)

                if i3 == 0 or i3 == degree - 1:
                    every = every + tipnum - 1
                else:
                    every = every + tipnum - 2
                #l = lx
            k = k + 1
            l = []
            j = 1
            k = 0
            w4 = 0
            f.write('\n')
        elif w5 == 1:
            l1 = re.findall(r'[0-9]+', line)
            if len(l1) == 6:
                l.append(l1)
                if eval(l1[0]) == tipnum5:
                    w5 = 2
                    j = 1
            #lx = l
        elif w5 == 2:
            every = 0
            for i3 in range(degree):
                repeated = []
                for i4 in range(len(l)):
                    if i3 == 0:
                        if eval(l[i4][2]) == xmaxnum:
                            continue
                        elif eval(l[i4][2]) > xmaxnum:
                            l[i4][2] = str(eval(l[i4][2]) - 1)
                        if eval(l[i4][3]) == xmaxnum:
                            if eval(l[i4][2]) >= xmaxnum:
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            continue
                        elif eval(l[i4][3]) > xmaxnum:
                            l[i4][3] = str(eval(l[i4][3]) - 1)
                        if eval(l[i4][4]) == xmaxnum:
                            if eval(l[i4][2]) >= xmaxnum:
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            if eval(l[i4][3]) >= xmaxnum:
                                l[i4][3] = str(eval(l[i4][3]) + 1)
                            continue
                        elif eval(l[i4][4]) > xmaxnum:
                            l[i4][4] = str(eval(l[i4][4]) - 1)
                        if eval(l[i4][5]) == xmaxnum:
                            if eval(l[i4][2]) >= xmaxnum:
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            if eval(l[i4][3]) >= xmaxnum:
                                l[i4][3] = str(eval(l[i4][3]) + 1)
                            if eval(l[i4][4]) >= xmaxnum:
                                l[i4][4] = str(eval(l[i4][4]) + 1)
                            continue
                        elif eval(l[i4][5]) > xmaxnum:
                            l[i4][5] = str(eval(l[i4][5]) - 1)
                    elif i3 == degree-1:
                        if eval(l[i4][2]) == xminnum:
                            continue
                        elif eval(l[i4][2]) > xminnum:
                            l[i4][2] = str(eval(l[i4][2]) - 1)
                        if eval(l[i4][3]) == xminnum:
                            if eval(l[i4][2]) >= xminnum:
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            continue
                        elif eval(l[i4][3]) > xminnum:
                            l[i4][3] = str(eval(l[i4][3]) - 1)
                        if eval(l[i4][4]) == xminnum:
                            if eval(l[i4][2]) >= xminnum:
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            if eval(l[i4][3]) >= xminnum:
                                l[i4][3] = str(eval(l[i4][3]) + 1)
                            continue
                        elif eval(l[i4][4]) > xminnum:
                            l[i4][4] = str(eval(l[i4][4])-1)
                        if eval(l[i4][5]) == xminnum:
                            if eval(l[i4][2]) >= xminnum:
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            if eval(l[i4][3]) >= xminnum:
                                l[i4][3] = str(eval(l[i4][3]) + 1)
                            if eval(l[i4][4]) >= xminnum:
                                l[i4][4] = str(eval(l[i4][4]) + 1)
                            continue
                        elif eval(l[i4][5]) > xminnum:
                            l[i4][5] = str(eval(l[i4][5])-1)
                    else:
                        if eval(l[i4][2]) in [xminnum, xmaxnum]:
                            continue
                        elif eval(l[i4][2])>min([xminnum, xmaxnum]):
                            l[i4][2] = str(eval(l[i4][2])-1)
                        elif eval(l[i4][2])>max([xminnum, xmaxnum]):
                            l[i4][2] = str(eval(l[i4][2])-2)  
                        if eval(l[i4][3]) in [xminnum, xmaxnum]:
                            if eval(l[i4][2]) >= min([xminnum, xmaxnum]):
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            elif eval(l[i4][2]) + 1 >= max([xminnum, xmaxnum]):
                                l[i4][2] = str(eval(l[i4][2]) + 2)
                            continue
                        elif eval(l[i4][3])>min([xminnum, xmaxnum]):
                            l[i4][3] = str(eval(l[i4][3])-1)
                        elif eval(l[i4][3])>max([xminnum, xmaxnum]):
                            l[i4][3] = str(eval(l[i4][3])-2)  
                        if eval(l[i4][4]) in [xminnum, xmaxnum]:
                            if eval(l[i4][2]) >= min([xminnum, xmaxnum]):
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            elif eval(l[i4][2]) + 1 >= max([xminnum, xmaxnum]):
                                l[i4][2] = str(eval(l[i4][2]) + 2)
                            if eval(l[i4][3]) >= min([xminnum, xmaxnum]):
                                l[i4][3] = str(eval(l[i4][3]) + 1)
                            elif eval(l[i4][3]) + 1 >= max([xminnum, xmaxnum]):
                                l[i4][3] = str(eval(l[i4][3]) + 2)
                            continue
                        elif eval(l[i4][4])>min([xminnum, xmaxnum]):
                            l[i4][4] = str(eval(l[i4][4]) - 1)
                        elif eval(l[i4][4])>max([xminnum, xmaxnum]):
                            l[i4][4] = str(eval(l[i4][4]) - 2)  
                        if eval(l[i4][5]) in [xminnum, xmaxnum]:
                            if eval(l[i4][2]) >= min([xminnum, xmaxnum]):
                                l[i4][2] = str(eval(l[i4][2]) + 1)
                            elif eval(l[i4][2]) + 1 >= max([xminnum, xmaxnum]):
                                l[i4][2] = str(eval(l[i4][2]) + 2)
                            if eval(l[i4][3]) >= min([xminnum, xmaxnum]):
                                l[i4][3] = str(eval(l[i4][3]) + 1)
                            elif eval(l[i4][3]) + 1 >= max([xminnum, xmaxnum]):
                                l[i4][3] = str(eval(l[i4][3]) + 2)
                            if eval(l[i4][4]) >= min([xminnum, xmaxnum]):
                                l[i4][4] = str(eval(l[i4][4]) + 1)
                            elif eval(l[i4][4]) + 1 >= max([xminnum, xmaxnum]):
                                l[i4][4] = str(eval(l[i4][4]) + 2)
                            continue
                        elif eval(l[i4][5])>min([xminnum, xmaxnum]):
                            l[i4][5] = str(eval(l[i4][5])-1)
                        elif eval(l[i4][5])>max([xminnum, xmaxnum]):
                            l[i4][5] = str(eval(l[i4][5])-2)
                    f.write('{0} {1} {2} {3} {4} {5}\n'.format(j, eval(l[i4][1]),\
                                                               eval(l[i4][2]) + every, eval(l[i4][3]) + every,\
                                                               eval(l[i4][4]) + every, eval(l[i4][5]) + every))
                    j = j + 1
                for i4 in range(len(l)):
                    if i3 == 0:
                        if (([eval(l[i4][2]), eval(l[i4][3]),\
                              eval(l[i4][4]), eval(l[i4][5])] in lost5_u
                                  or [eval(l[i4][2]), eval(l[i4][3]),\
                                      eval(l[i4][4]), eval(l[i4][5])] in lost5_d) and
                            [eval(l[i4][2]), eval(l[i4][3]),\
                             eval(l[i4][4]), eval(l[i4][5])] not in repeated):
                            repeated.append([eval(l[i4][2]), eval(l[i4][3]),\
                                             eval(l[i4][4]), eval(l[i4][5])])
                            continue
                        if eval(l[i4][2]) >= xmaxnum:
                            l[i4][2] = str(eval(l[i4][2]) + 1)
                        if eval(l[i4][3]) >= xmaxnum:
                            l[i4][3] = str(eval(l[i4][3]) + 1)
                        if eval(l[i4][4]) >= xmaxnum:
                            l[i4][4] = str(eval(l[i4][4]) + 1)
                        if eval(l[i4][5]) >= xmaxnum:
                            l[i4][5] = str(eval(l[i4][5]) + 1)
                    elif i3 == degree - 1:
                        if (([eval(l[i4][2]), eval(l[i4][3]),\
                              eval(l[i4][4]), eval(l[i4][5])] in lost5_u
                                 or [eval(l[i4][2]), eval(l[i4][3]),\
                                     eval(l[i4][4]), eval(l[i4][5])] in lost5_d) and
                            [eval(l[i4][2]), eval(l[i4][3]),\
                             eval(l[i4][4]), eval(l[i4][5])] not in repeated):
                            repeated.append([eval(l[i4][2]), eval(l[i4][3]),\
                                             eval(l[i4][4]), eval(l[i4][5])])
                            continue
                        if eval(l[i4][2]) >= xminnum:
                            l[i4][2] = str(eval(l[i4][2]) + 1)
                        if eval(l[i4][3]) >= xminnum:
                            l[i4][3] = str(eval(l[i4][3]) + 1)
                        if eval(l[i4][4]) >= xminnum:
                            l[i4][4] = str(eval(l[i4][4]) + 1)
                        if eval(l[i4][5]) >= xminnum:
                            l[i4][5] = str(eval(l[i4][5]) + 1)
                    else:
                        if (([eval(l[i4][2]), eval(l[i4][3]),\
                              eval(l[i4][4]), eval(l[i4][5])] in lost5_u
                                 or [eval(l[i4][2]), eval(l[i4][3]),\
                                     eval(l[i4][4]), eval(l[i4][5])] in lost5_d) and
                            [eval(l[i4][2]), eval(l[i4][3]),\
                             eval(l[i4][4]), eval(l[i4][5])] not in repeated):
                            repeated.append([eval(l[i4][2]), eval(l[i4][3]),\
                                             eval(l[i4][4]), eval(l[i4][5])])
                            continue
                        if eval(l[i4][2]) >= min([xminnum, xmaxnum]):
                            l[i4][2] = str(eval(l[i4][2]) + 1)
                        elif eval(l[i4][2]) + 1 >= max([xminnum, xmaxnum]):
                            l[i4][2] = str(eval(l[i4][2]) + 2)  
                        if eval(l[i4][3]) >= min([xminnum, xmaxnum]):
                            l[i4][3] = str(eval(l[i4][3]) + 1)
                        elif eval(l[i4][3]) + 1 >= max([xminnum, xmaxnum]):
                            l[i4][3] = str(eval(l[i4][3]) + 2)  
                        if eval(l[i4][4]) >= min([xminnum, xmaxnum]):
                            l[i4][4] = str(eval(l[i4][4]) + 1)
                        elif eval(l[i4][4]) + 1 >= max([xminnum, xmaxnum]):
                            l[i4][4] = str(eval(l[i4][4]) + 2)
                        if eval(l[i4][5]) >= min([xminnum, xmaxnum]):
                            l[i4][5] = str(eval(l[i4][5]) + 1)
                        elif eval(l[i4][5]) + 1 >= max([xminnum, xmaxnum]):
                            l[i4][5] = str(eval(l[i4][5]) + 2)
                if i3 == 0 or i3 == degree - 1:
                    every = every + tipnum - 1
                else:
                    every = every + tipnum - 2
                #lx = l
            k = k + 1
            l = []
            j = 1
            k = 0
            w4 = 0
            l = lx
            f.write('\n')
        else:
            f.write(line)
    f.close()
    inp.close()
    med.close()
    os.chdir(opath)


import os
import re
import warnings
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
from .YModulus import YModulus

def multiple(allname='Result', polymers=[], loc='./', savefig=True,\
             savedata=True, needYM=True, xx=False):
    """
    The method to create a Stress-Strain Curve chart and .csv relevant data file of all polymers you choose.
    multiple(allname='Results', polymers=[file1, file2, file3, ...], loc='./', savefig=True, savedata=True, needYM=True))
    allname: The name of your saved Stress-Strain curve chart and data file.
    polymers: Your selected polymer names on your file. The default is all polymers which has _Results.txt file in the location. 
    loc: File Location. The default is your current location.
    savefig, savedata: To show if you want to save the chart and .csv curve. The default is True.
    neddYM: To calculate Young's modulus of each polymers and display it in the data file.
    You can get the further information by .curve and .autocurve.
    Example:
        Input:
            from MCPoly.sscurve import multiple
            multiple(['P1','P2'])
        Output:
            ['P1','P2']
                               Strain Length (P1)  Strain Length (P2)  \
            Stress Force(nN)                                            
            0.0                         1.000000             1.000000   
            0.5                         1.012022                  NaN   
            1.0                         1.024666             1.028089   
            1.5                              NaN                  NaN   
            2.0                         1.053014             1.058092   
            2.5                              NaN                  NaN   
            3.0                         1.086888             1.093080   
            3.5                              NaN                  NaN   
            4.0                         1.130600             1.136959   
            4.5                              NaN                  NaN   
            5.0                         1.199289             1.203102   
            5.5                              NaN                  NaN   
            6.0                              NaN                  NaN   
            6.5                              NaN                  NaN   
            7.0                              NaN                  NaN   
            7.5                              NaN                  NaN   
            8.0                              NaN                  NaN   
            8.5                              NaN                  NaN   
            9.0                              NaN                  NaN   
            9.5                              NaN                  NaN   
            10.0                             NaN                  NaN   
            NaN                        29.516401             23.478160  NaN-> Young's Modulus
    TIPS: This function only depends on _Results.txt file, not .mol, .inp, .out, .xyz files.
    """
    opath = os.getcwd()
    if polymers == []:
        for path in os.listdir(loc):
            if os.path.isfile(os.path.join(loc, path)):
                a = re.search('_Result.txt', path)
                if a:
                    polymers.append(path[:-11])            
    print(polymers)
    
    os.chdir(loc)
    try:
        f = open('ot.txt','x')
    except:
        f = open('ot.txt','w')
    
    ym = [None]
    name = []
    i = 0
    
    site = []

    for polymer in polymers:
        polymer = polymer + '_Result.txt'
        try:
            datum = pd.read_csv(polymer)
        except:
            warnings.warn("'{0}' is not found.".format(polymer))
            continue
        try:
            datum = datum[:].sort_values(by=['Strain Length(%)'])
        except:
            warnings.warn('{0} is not the file about'.format(polymer)
                          + 'stress-strain curve.')
            continue
        for num in datum['Stress Force(nN)'][:-1]:
            if num not in site:
                site.append(num)
    site.sort()
    f.write('Stress Force(nN)\n')
    for num in site:
        f.write('{0:.3f}\n'.format(num))
    f.close()
    
    ot = pd.read_csv('ot.txt')
    for polymer in polymers:
        polymer = polymer + '_Result.txt'
        try:
            datum = pd.read_csv(polymer)
        except:
            warnings.warn("'{0}' is not found.".format(polymer))
            continue
        datum = datum[:].sort_values(by=['Strain Length(%)'])
        if xx == False:
            ax = plt.plot(datum['Strain Length(%)'][:-1],\
                          datum['Stress Force(nN)'][:-1],'*-')
        if needYM == True:
            ym.append(YModulus(polymer))
        datum = datum.rename(columns={
            'Strain Length(%)':'Strain Length ({0})'.format(polymer[:-11])})
        ot = pd.merge_ordered(ot, datum[:-1], fill_method="ffill",\
                              left_by="Stress Force(nN)")
        name.append(polymer[:-11])
        i = i + 1
    
    if needYM == True:
        al = pd.DataFrame(data=ym, index=ot.columns, columns=["Young's Modulus"])
        al = al.T
        #print(al)
        ot = pd.concat([ot,al])
    ot = ot.set_index('Stress Force(nN)')
    display(ot)
    if savedata == True:
        ot.to_csv('{0}.csv'.format(allname))
    
    if xx == False:
        #ax = sns.scatterplot(data=ot, alpha=0.8)
        #ax.set_ylabels = 'Strain Length(%)'
        #_ = plt.title('Stress-Strain Curve')
        #bx = sns.relplot(data=ot, kind="line", alpha=0.5)
        #bx.set_ylabels = 'Strain Length(%)'
        #ax.legend(bbox_to_anchor=[1.1,0.9])
        plt.xlabel('Strain Length(%)')
        plt.ylabel('Stress Force(nN)')
        plt.legend(name,bbox_to_anchor=[1.1,0.9])
        _ = plt.title('Stress-Strain Curve')
        
    if savefig == True:
        plt.savefig('{0}.png'.format(allname), bbox_inches='tight')
    if xx == False:
        plt.show()
    
    os.system('rm ot.txt')
    os.chdir(opath)


import os
import warnings
from ase.io import read

def XyzToInp(file, inpname='', fileloc='./', saveloc='./', method='B3LYP',\
             basis_set='def2-SVP', opt=False, freq=False, scan=False,\
             external_force=False, ts=False, aim=[0, 0], stretch=-1, scanstep=10,\
             strain=-1, maxiter=-1, maxcore=-1, corenum=1, electron=0, state=1,\
             MD=False, md_timestep=1.0, initvel=300, tsNHC=300, timecon=10.0,\
             dumpcon=10, runstep=100, dumpname=''):
    """
    The method to convert .xyz file into .inp file.
    XyzToInp(file, inpname='', loc='./', method='B3LYP', basis_set='def2-SVP', opt=False, freq=False, scan=False, external_force=False, ts=False, aim=[0, 0], stretch=1, scanstep=10, strain=-1, maxiter=-1, maxcore=-1, corenum=1, electron=0, state=1)
    file: Your .xyz file name.
    inpname: The name of your saved .inp file.
    fileloc: File Location. The default is your current location.
    saveloc: Input File Save Location. The default is your current location.
    method: Your semiempirical/ab initio/DFT calculation methods. The default is B3LYP.
    basis_set: Your basis set. The default is def2-SVP.
    opt: To show if you want to optimise your molecule. The default is False.
    freq: To show if you want to calculate the frequency of intermolecule system and thermodynamic data. The default is False.
    scan: To show if you want to scan the system. You must use key word 'aim', 'stretch' and 'scanstep' to express the site of external force, or it will cause 'SetError'. 
        Herein, 'aim' is the site of external force, 'stretch' is the length you need to stretch (>0) or shrink(<0). (Unit: Å), and 'scanstep' is the number of your scan steps.
        The default is False.
    external_force: To show if you want to impose external force to the system. You must use key word 'aim' to express the site of external force AND key word 'strain' to express the force energy (Unit: nN), or it will cause 'ValueError'. The default is False.
    ts: To show if you want to calculate the transition state of the system. The default is False.
    maxiter: The max iteration steps of the optimization. It must perform after key word 'optimise' is True, or it will cause 'ValueError'. The default is based on ORCA.
    maxcore: The max space you need to calculate. The default is based on ORCA.
    corenum: The core you need to calculate. The default is 1.
    TIPS: To use key word 'maxcore' and 'corenum', please make sure you have set ORCA from following two websites.
            https://www.orcasoftware.de/tutorials_orca/first_steps/trouble_install.html#path-variable
            https://www.orcasoftware.de/tutorials_orca/first_steps/parallel.html
    electron: The charge of your system. It might be positive(>0), negative(<0) or neutral(=0). The default is neutral.
    state: The state of your system (singlet:1, doublet:2, triplet:3 ...). The default is singlet.
    Example 1:
        Input:
            from MCPoly.orcaset import XYZtoINP
            XyzToInp('Atom1')
        Output in Atom1.inp:
            #Powered by MCPoly

            ! B3LYP def2-SVP 

            *xyz 0 1
            C         -7.11449        1.33604       -0.03969
            C         -5.87285        0.46952       -0.06689
            C         -4.65267        1.33725        0.06683
            C         -3.41103        0.47072        0.03973
            F         -7.23198        2.03119        1.11508
                ...
            *
            
            
    Example 2:
        Input:
            from MCPoly.orcaset import XYZtoINP
            XyzToInp('Atom1', method='B3LYP D3BJ', basis_set='def2-TZVP', freq=True, \
                     external_force=True, aim=[0, 3], strain=1.5, maxcore=4096, corenum=8)
        Output in Atom1.inp:
            #Powered by MCPoly
            
            ! B3LYP D3BJ def2-TZVP OPT FREQ
            %maxcore 4096
            %PAL NPROCS 8 END
            %geom
            POTENTIALS
            {C 0 3 1.500}
            end
            end
            
            *xyz 0 1
            C         -7.11449        1.33604       -0.03969
            C         -5.87285        0.46952       -0.06689
            C         -4.65267        1.33725        0.06683
            C         -3.41103        0.47072        0.03973
            F         -7.23198        2.03119        1.11508
                ...
            *
    
    
    Example 3:
        Input:
            from MCPoly.orcaset import XYZtoINP
            XyzToInp('Atom1', method='B3LYP D3BJ', basis_set='def2-TZVP', scan=True, \
                     aim=[0, 3], stretch=0.8, scanstep=8, maxcore=4096, corenum=8)
        Output in Atom1.inp:
            #Powered by MCPoly
            
            ! B3LYP D3BJ def2-TZVP 
            %maxcore 4096
            %PAL NPROCS 8 END
            %geom Scan
            B 0 3 = 3.804, 4.604, 8
            end
            end
            
            *xyz 0 1
            C         -7.11449        1.33604       -0.03969
            C         -5.87285        0.46952       -0.06689
            C         -4.65267        1.33725        0.06683
            C         -3.41103        0.47072        0.03973
            F         -7.23198        2.03119        1.11508
                ...
            *
    
    Example 4:
        Input:
            from MCPoly.orcaset import XYZtoINP
            XyzToInp('Atom1', method='B3LYP D3BJ', basis_set='def2-TZVP', MD=True, \
                     MD=True, freq=True, md_timestep=1.0, initvel=350, tsNHC=350, dumpcon=25, runstep=2500, maxcore=4096, corenum=8)
        Output in Atom1.inp:
            #Powered by MCPoly
            
            ! MD B3LYP D3BJ def2-TZVP 
            %md
            Timestep 1.0_fs
            Initvel 350_K
            Thermostat NHC 350_K Timecon 10.0_fs
            Dump Position Stride 25 Filename "S4A_dump.xyz"
            Run 2500
            end
            %maxcore 4096
            %PAL NPROCS 8 END
            
            *xyz 0 1
            C         -7.11449        1.33604       -0.03969
            C         -5.87285        0.46952       -0.06689
            C         -4.65267        1.33725        0.06683
            C         -3.41103        0.47072        0.03973
            F         -7.23198        2.03119        1.11508
                ...
            *
    """
    f = open(fileloc+file+'.xyz', 'r')
    if inpname == '':
        try:
            w = open(saveloc+file+'.inp', 'x')
        except:
            w = open(saveloc+file+'.inp', 'w')
    else:
        try:
            w = open(saveloc+inpname+'.inp', 'x')
        except:
            w = open(saveloc+inpname+'.inp', 'w')
    w.write('#Powered by MCPoly\n\n')
    w.write('! ')
    if MD == True:
        w.write('MD ')
        sacn = False
        opt = False
        freq = False
        external_force = False
    
    w.write('{0} {1} '.format(method, basis_set))
        
    if MD == True:
        w.write('\n')
        if initvel < 0:
            raise ValueError("Key word 'initvel' must be positive,\
                for the unit is Kelvin.")
        if tsNHC < 0:
            raise ValueError("Key word 'tsNHC' must be positive,\
                for the unit is Kelvin.")
        w.write('%md\n')
        w.write('Timestep {0}_fs\n'.format(md_timestep))
        w.write('Initvel {0}_K\n'.format(initvel))
        w.write('Thermostat NHC {0}_K Timecon {1}_fs\n'.format(tsNHC, timecon))
        if dumpname == '':
            w.write('Dump Position Stride {0} '.format(dumpcon)
                    + 'Filename "{0}_dump.xyz"\n'.format(file))
        else:
            w.write('Dump Position Stride {0}'.format(dumpcon)
                    + 'Filename "{0}_dump.xyz"\n'.format(dumpname))
        w.write('Run {0}\n'.format(runstep))
        w.write('end\n')
        
    if opt == True:
        if scan == True:
            pass
        if ts == True:
            w.write('OPTTS ')
        else:
            w.write('OPT ')
    if external_force == True and opt == False:
        w.write('OPT ')
    if freq == True:
        if opt == False:
            warnings.warn('Your set the frequency calculation'
                          + ' without geometry optimisation.')
        w.write('FREQ')
    w.write('\n')
    
    if maxiter != -1:
        if maxiter <= 0 or type(maxiter) == float:
            raise ValueError("Key word 'maxiter' must be a positive integar.")
        w.write('%geom\n    MAXITER {0}\nend\n'.format(maxiter))
    
    if maxcore != -1:
        w.write('%maxcore {0}\n'.format(maxcore))
        if maxcore <= 2048:
            warnings.warn('Your max core space is only {0:.3f} Å.'.format(maxcore)
                + 'You might delete this key words to get the same calculation speed,'
                + 'or try bigger one.')

    if corenum != 1:
        w.write('%PAL NPROCS {0} END\n'.format(corenum))
    
    if scan == True:
        if aim == [0, 0]:
            raise ValueError("You haven't set the site of two atoms you scan.")
        elif (aim[0] < 0 or aim[1] < 0 or 
              type(aim[0]) == float or type(aim[1]) == float):
            raise ValueError("The site of two atoms must be not negative integar.")
        elif aim[0] == aim[1]:
            raise ValueError("You can't scan on the same two atoms.")
        if scanstep < 0 or type(scanstep) == float:
            raise ValueError("The scan steps must be positive integar.")
        if ts == True:
            raise AssertionError("You can't set scan and"
                                 + "transition state at the same time.")
        if freq == True:
            raise AssertionError("You can't set scan and"
                                 + "frequency calculation at the same time.")
        atoms = read(fileloc+file+'.xyz')
        distance = atoms.get_distance(aim[0], aim[1])
        if distance + stretch < 0:
            raise ValueError("You can't set the final distance"
                             + "of atoms less than 0.")
        elif distance + stretch <= 0.8:
            warnings.warn('Your final distance of two atoms'
                          + 'is only {0:.3f} Å.'.format(distance+stretch))
        w.write('%geom Scan\n')
        w.write('B {0} {1} = {2:.3f},  {3:.3f},  {4}\nend\nend\n'.format(*aim,\
                                                                       distance, distance+stretch, scanstep))

    if external_force == True:
        if aim == [0, 0]:
            raise ValueError("You haven't set the site of two atoms you scan.")
        elif (aim[0] < 0 or aim[1] < 0 or
              type(aim[0]) == float or type(aim[1]) == float):
            raise ValueError("The site of two atoms must be not negative integar.")
        elif aim[0] == aim[1]:
            raise ValueError("You can't scan on the same two atoms.")
        if scan == True:
            raise AssertionError("You can't set scan and external force at the same time.")
        if ts == True:
            raise AssertionError("You can't set external force"
                                 + "and transition state at the same time.")
        w.write('%geom\nPOTENTIALS\n')
        w.write('{C ' + '{0} {1} {2:.3f}'.format(*aim, strain) + '}\nend\nend\n')

    w.write('\n*xyz {0} {1}\n'.format(electron, state))
    i = 0
    for line in f:
        i = i + 1
        if i >= 3:
            w.write(line)
    w.write('*\n')
    f.close()
    w.close()


import os
import re
import sys
from .XyzToInp import XyzToInp
from .orca import orca
from ase.io import read
import time as t

mydir = os.path.dirname( __file__ )
statusdir = os.path.join(mydir, '..', 'status')
sys.path.append(statusdir)
from status import status

def checkbroken(distance, force):
    long1 = abs(distance[-2] - distance[-1])
    long2 = abs(distance[-3] - distance[-2])
    mul1 = abs(force[-2] - force[-1])
    mul2 = abs(force[-3] - force[-2])
    if long1 / long2 >= 10 * mul1 / mul2:
        return 1
    else:
        return 0

class ssorca:
    """
    The method to calculate .inp file with growing external force in one document consecutively by ORCA.
    ssorca(file, loc)
    file: Your .xyz file name.
    loc: File Location. The default is your current location.
    You can get the further information by .run.
    """
    def __init__(self, file, loc='./'):
        self.loc=loc
        self.file=file
        
    def run(self, orcaloc='./', method='B3LYP', basis_set='def2-SVP', freq=False,\
            aim=[0, 0], strain=0, forcestep={0:1}, maxiter=-1,\
            maxcore=-1, corenum=1, electron=0, state=1):
        """
    The method to calculate .inp file with growing external force by ORCA and save file at the same location with .inp files, powered by ase.
    run(self, orcaloc='./', method='B3LYP', basis_set='def2-SVP', freq=False, external_force=False, aim=[0, 0], strain=0, maxiter=-1, maxcore=-1, corenum=1, electron=0, state=1)
    orcaloc: Your location of ORCA.
             If you have set ORCA from https://www.orcasoftware.de/tutorials_orca/first_steps/trouble_install.html#path-variable, 
             you can skip this key word.
    method: Your semiempirical/ab initio/DFT calculation methods. The default is B3LYP.
    basis_set: Your basis sets. The default is def2-SVP.
    opt: To show if you want to optimise your molecule. The default is False.
    freq: To show if you want to calculate the frequency of intermolecule system and thermodynamic data. The default is False.
    aim: To express two sites of external force.
    strain: To express the beginning force energy (Unit: nN). The default is no external force.
    forcestep: The external force grow per step. You can input multiple external force grow. 
               e.g. {0:1, 5:0.5} -> from 0 nN to 5 nN, it grows 1 nN per step; when > 5 nN, it grows 0.5 nN per step.
               The default is {0:1}, growing 1 nN per step always.
    TIPS: 1. You needn't rectify if forcestep={0:0.7, 5:0.5}, thought 5.0 nN will not be calculated.
          2. The accuracy of strain and forcestep is 0.001.
    maxiter: The max iteration steps of the optimization. It must perform after key word 'optimise' is True, or it will cause 'ValueError'. The default is based on ORCA.
    maxcore: The max space you need to calculate. The default is based on ORCA.
    corenum: The core you need to calculate. The default is 1.
    TIPS: To use key word 'maxcore' and 'corenum', please make sure you have set ORCA from following two websites.
            https://www.orcasoftware.de/tutorials_orca/first_steps/trouble_install.html#path-variable
            https://www.orcasoftware.de/tutorials_orca/first_steps/parallel.html
    electron: The charge of your systems. It might be positive(>0), negative(<0) or neutral(=0). The default is neutral.
    state: The state of your systems (singlet:1, doublet:2, triplet:3 ...). The default is singlet.
    TIPS: 1. Don't put the .inp to your ORCA installment file, it may disturb your work.     
          2. If you don't input key word 'orcaloc' while you don't set ORCA, you will only get an empty .out file in the document.
             There will be a warning: sh: orca: 'command not found'.
          3. If you have set key word 'maxcore' and 'corenum', you must input 'orcaloc'.
             See in https://www.orcasoftware.de/tutorials_orca/first_steps/parallel.html
          4. You can see the error of ORCA directly from the output.
    Example 1:
        Input:
            from MCPoly.orcaset import ssorca
            polymer = ssorca('Atom1')
            orcaloc = './MCPoly/orca/'
            polymer.run(orcaloc=orcaloc, method='B3LYP', basis_set='def2-TZVP', freq=True, aim=[2, 3], forcestep={0:0.5},\
                        maxcore=4096, corenum=8)
        Output:
            File: Atom1.xyz
            Current External Force: 0.000 nN [Tue Apr 25 15:23:49 2023]
            <<COMMAND LINES>>
            Program Completed.[Tue Apr 25 15:43:58 2023]
            
            Current External Force: 0.500 nN [Tue Apr 25 15:43:59 2023]
            <<COMMAND LINES>>
            Program Completed.[Tue Apr 25 15:58:12 2023]
            
            Current External Force: 1.000 nN [Tue Apr 25 15:58:12 2023]
            <<COMMAND LINES>>
            Program Completed.[Tue Apr 25 16:13:34 2023]
            
            Current External Force: 1.500 nN [Tue Apr 25 16:13:34 2023]
            <<COMMAND LINES>>
            Program Completed.[Tue Apr 25 16:39:47 2023]
            
            The polymer is broken. Programme Completed.[Tue Apr 25 16:39:47 2023]
            [file orca_main/run.cpp, line 32327]: ORCA finished with error return - aborting the run     -> COMMAND LINES

    Example 2:
        Input:
            from MCPoly.orcaset import ssorca
            polymer = ssorca('Atom2')
            orcaloc = './MCPoly/orca/'
            polymer.run(orcaloc=orcaloc, method='B3LYP', basis_set='def2-TZVP', freq=True, aim=[2, 3], forcestep={0:1, 5:0.5},\
                        maxcore=4096, corenum=8)
        Output:
            File: Atom2.xyz
            Current External Force: 0.000 nN [Tue Apr 25 15:23:49 2023]
            <<COMMAND LINES>>
            Program Completed.[Tue Apr 25 15:43:52 2023]
            
            Current External Force: 1.000 nN [Tue Apr 25 15:43:52 2023]
            <<COMMAND LINES>>
            Program Completed.[Tue Apr 25 15:58:12 2023]
            
            ...
            
            Current External Force: 5.000 nN [Tue Apr 25 17:03:26 2023]
            <<COMMAND LINES>>
            Program Completed.[Tue Apr 25 17:12:03 2023]
            
            Current External Force: 5.500 nN [Tue Apr 25 17:12:03 2023]
            <<COMMAND LINES>>
            Program Completed.[Tue Apr 25 17:24:39 2023]
            
            Current External Force: 6.000 nN [Tue Apr 25 17:24:40 2023]
            <<COMMAND LINES>>
            Program Completed.[Tue Apr 25 17:51:31 2023]
            
            The polymer is broken. Programme Completed.[Tue Apr 25 17:51:31 2023]
            [file orca_main/run.cpp, line 73453]: ORCA finished with error return - aborting the run     -> COMMAND LINES
        """
        distances = []
        force = []
        print('File: {0}.xyz'.format(self.file))
        XyzToInp(self.file, fileloc=self.loc, method=method,\
                 basis_set=basis_set, opt=True, freq=freq, \
                 external_force=True, aim=aim, strain=strain,\
                 maxiter=maxiter, maxcore=maxcore, corenum=corenum, \
                 electron=electron, state=state)
        os.system('cp {0}.inp {0}_0.000.inp'.format(self.file))
        keys = []
        for key in forcestep:
            keys.append(key)
        keys.reverse()
        key = keys[0]
        while 1:
            print('Current External Force: {0:.3f} nN [{1}]'.format(strain,\
                                                                    t.ctime(t.time())))
            orca(self.file+'_{0:.3f}'.format(strain), orcaloc, self.loc, self.loc)
            s = status(self.file+'_{0:.3f}'.format(strain),\
                       self.loc).status(statusonly=True)
            atoms = read(file+'_{0:.3f}.xyz'.format(strain))
            d = atoms.get_distance(aim[0], aim[1])
            if s == 4:
                distances.append(d)
                force.append(strain)
                xoy = checkbroken(distances, force)
                if xoy == 1:
                    print('The polymer is broken.\
                        Programme Completed. [{0}]'.format(t.ctime(t.time())))
                    break
                print('This calculation has some errors.\nThe programme\
                    will recalculate this system again. [{0}]\n'.format(t.ctime(t.time())))
                orca(self.file+'_{0:.3f}'.format(strain), orcaloc, self.loc, self.loc)
                s = status(self.file, self.loc).status(statusonly=True)
                if s == 4:
                    print('This calculation still has some errors.\n\
                        Please check\
                        your .inp and try again. [{0}]\n'.format(t.ctime(t.time())))
                    distances[-1] = None
                else:
                    distances[-1] = d
            else:
                distances.append(d)
                force.append(strain)
            for key in keys:
                if strain >= key:
                    XyzToInp(self.file+'_{0:.3f}'.format(strain),\
                             inpname=self.file+'_{0:.3f}'.format(strain+forcestep[key]), \
                             method=method, basis_set=basis_set, opt=True, freq=freq, \
                             external_force=True, aim=aim, strain = strain + forcestep[key],\
                             maxiter=maxiter, maxcore=maxcore,\
                             corenum=corenum, electron=electron, state=state)
                    strain = strain + forcestep[key]
                    break


# import numpy as np
import os
import re
import py3Dmol
from ase.io import read, write
from ase import Atoms
from ase.visualize import view
from MCPoly import view3d
from MCPoly.moldraw import molecule
from MCPoly.orcaset import view3dchoose
import ipywidgets as widgets
from IPython.display import display
from rdkit import Chem
from rdkit.Chem import AllChem
from ipywidgets import interact

def smi2conf(smiles):
    '''Convert SMILES to rdkit.Mol with 3D coordinates'''
    mol = Chem.MolFromSmiles(smiles)
    if mol is not None:
        mol = Chem.AddHs(mol)
        AllChem.EmbedMolecule(mol)
        return mol
    else:
        return None

def MolTo3DView(mol, size=(400, 550), surface=False, opacity=0.5):
    """Draw molecule in 3D (Not Origin)
    
    Args:
    ----
        mol: rdMol, molecule to show
        size: tuple(int, int), canvas size
        style: str, type of drawing molecule
               style can be 'line', 'stick', 'sphere', 'cartoon'
        surface, bool, display SAS
        opacity, float, opacity of surface, range 0.0-1.0
    Return:
    ----
        viewer: py3Dmol.view, a class for constructing embedded 3Dmol.js views in ipython notebooks.
    """
    mblock = Chem.MolToMolBlock(mol)
    viewer = py3Dmol.view(width=size[0], height=size[1])
    viewer.addModel(mblock, 'mol')
    viewer.setStyle({'sphere':{'radius':0.4}, 'stick':{'radius':0.1}})
    if surface:
        viewer.addSurface(py3Dmol.SAS, {'opacity': opacity})
    viewer.zoomTo()
    Chem.MolToXYZFile(mol, 'Mole_Untitled.xyz')
    return viewer

def gui():
    """
        The method to draw molecule and save it in the folder.
        You will know how to do as long as you get into the GUI platform.
    
        TIPS: 1. This drawing function is under construction. There might be some errors in the programme.
              2. 'shuffle', 'less than 5' are used to modify some geometry structure.
              3. The visualisation will also risk the flexibility of the task, to make you programme more flexible, please directly use the code.
    """
    def changecombo(loc):
        options2 = []
        for path in os.listdir(loc):
            if os.path.isfile(os.path.join(loc, path)):
                a = re.search('.out', path)
                if a:
                    options2.append(path[:-4])
        options2.sort()
        file.options = options2
        
    def outputmol1(button):
        output.clear_output()
        output2.clear_output()
        placeoutput.clear_output()
        strictoutput1.clear_output()
        strictoutput2.clear_output()
        strictoutput3.clear_output()
        angleoutput.clear_output()
        shuffleoutput.clear_output()
        strictadd2.value=False
        strictadd3.value=False
        with output:
            i.max = atoms.atomnum() - 1
            if button.description == 'pyrrole':
                place.max = 5
                with placeoutput:
                    display(place)
            if button.description == 'Py':
                place.max = 6
                with placoutput:
                    display(place)
            if button.description == 'cyclo3':
                strictnum1.max = 3
            elif button.description == 'cyclo5':
                strictnum1.max = 5
                strictnum2.max = 5
            elif button.description == 'cyclo6':
                strictnum1.max = 6
                strictnum2.max = 6
                strictnum3.max = 6
            if button.description in ('C2H3', 'C=NH', 'Ph', 'Py', 'pyrrole', 'cyclo5',\
                                      'cyclo6', 'Bpin', 'bicyclo[2.2.1]', 'bicyclo[2.2.2]'):
                with shuffleoutput:
                    display(shuffle)
            if button.description not in ('cyclo3', 'cyclo5', 'cyclo6'):
                pass
            elif button.description == 'cyclo3':
                with strictoutput1:
                    display(widgets.VBox([strictnum1, widgets.HBox([strictele1])]))
            else:
                with strictoutput1:
                    display(widgets.VBox([strictnum1, widgets.HBox([strictele1, strictadd2])]))
            if button.description in ('OH', 'OMe', 'SH', 'SMe'):
                with angleoutput:
                    display(widgets.HBox([angle, angleavail]))
            buttonname.value=button.description
            display(widgets.VBox([i, state, placeoutput, strictoutput1,\
                                  strictoutput2, strictoutput3, shuffleoutput, angleoutput]))
            display(widgets.HBox([preview, settle]))
    
    def outputmol2(button):
        output.clear_output()
        output2.clear_output()
        placeoutput.clear_output()
        strictoutput1.clear_output()
        strictoutput2.clear_output()
        strictoutput3.clear_output()
        angleoutput.clear_output()
        shuffleoutput.clear_output()
        strictadd2.value = False
        strictadd3.value = False
        with output:
            i1.max = atoms.atomnum() - 1
            i2.max = atoms.atomnum() - 1
            state1.options = [('1', 1), ('2', 2), ('6', 6)]
            state2.options = [('1', 1), ('2', 2), ('6', 6)]
            if button.description == 'bind3':
                strictnum1.max = 1
            elif button.description in ('bind5', 'bindconj5'):
                strictnum1.max = 3
                strictnum2.max = 3
            elif button.description in ('bind6', 'bindconj6'):
                strictnum1.max = 4
                strictnum2.max = 4
            if button.description not in ('bind2', 'bindconj5', 'bindconj6'):
                with shuffleoutput:
                    display(shuffle)
            if button.description == 'bind2':
                pass
            elif button.description == 'bind3':
                with strictoutput1:
                    display(widgets.VBox([strictnum1, widgets.HBox([strictele1])]))
            else:
                with strictoutput1:
                    display(widgets.VBox([strictnum1, widgets.HBox([strictele1, strictadd2])]))
            buttonname.value = button.description
            display(widgets.VBox([widgets.HBox([i1, state1]), widgets.HBox([i2, state2]),\
                                  strictoutput1, strictoutput2, side, shuffle, less5, stable]))
            display(widgets.HBox([preview, settle]))
            
    def outputmol3(button):
        output.clear_output()
        with output:
            i.max = atoms.atomnum() - 1
            state1.options = [('1', 1), ('2', 2), ('6', 6), ('7', 7), ('8', 8), ('82', 82)]
            state2.options = [('1', 1), ('2', 2), ('6', 6), ('7', 7), ('8', 8), ('82', 82)]
            buttonname.value = button.description
            display(widgets.VBox([i, state1, state2]))
            display(widgets.HBox([preview, settle]))
        
    def clear(button):
        output.clear_output()
        output2.clear_output()

    def openf(button):
        os.system('cp {0}.xyz Mole_Untitled.xyz'.format(file.value))
        file.value = 'Mole_Untitled'
        with output3:
            view3d(file.value, loc.value, width=400, height=500)
            display(widgets.HBox([savename, savefile]))
        output0.clear_output()
        global atoms
        atoms = molecule(file.value, loc.value)
        with output0:
            display(widgets.VBox([allchoose, stack]))

    def smartshow(smart):
        os.system('rm Mole_Untitled.xyz')
        output3.clear_output()
        with output3:
            try:
                conf = smi2conf(smart)
                return MolTo3DView(conf).show()
            except:
                return None
    
    def smartset(button):
        output3.clear_output()
        with output3:
            view3d('Mole_Untitled', loc.value, width=400, height=500)
            display(widgets.HBox([savename, savefile]))
        output0.clear_output()
        file.value = 'Mole_Untitled'
        global atoms
        atoms = molecule('Mole_Untitled', loc.value)
        with output0:
            display(widgets.VBox([allchoose, stack]))
    
    def saveset(button):
        output0.clear_output()
        output.clear_output()
        output3.clear_output()
        atoms.xyzwrite(savename.value)
        os.system('rm Mole_Untitled.xyz')
        file.value = ''
        with output0:
            display(widgets.VBox([loc, widgets.HBox([file, openfile]),\
                                  widgets.Label('OR', style=dict(font_weight='bold',\
                                                                 text_decoration='underline')),\
                                  widgets.HBox([smart, smartbutton])]))
    
    def viewset(button):
        global atoms
        output3.clear_output()
        atoms_b = atoms
        if choose.value == 'Substrates':
            if buttonname.value == 'cyclo3':
                if strictele1.value[0] == 'X':
                    atoms_b.sub(buttonname.value, i.value, state.value,\
                                place.value, shuffle=shuffle.value, angle=angle.value)
                else:
                    atoms_b.sub(buttonname.value, i.value, state.value,\
                                place.value, strict={strictnum1:strictele1},\
                                shuffle=shuffle.value, angle=angle.value)
            elif buttonname.value == 'cyclo5':
                if strictele1.value[0] == 'X':
                    atoms_b.sub(buttonname.value, i.value, state.value, place.value,\
                                shuffle=shuffle.value, angle=angle.value)
                elif strictadd2.value == False:
                    atoms_b.sub(buttonname.value, i.value, state.value, place.value,\
                                strict={strictnum1:strictele1},\
                                shuffle=shuffle.value, angle=angle.value)
                else:
                    atoms_b.sub(buttonname.value, i.value, state.value, place.value,\
                                strict={strictnum1:strictele1, strictnum2:strictele2},\
                                shuffle=shuffle.value, angle=angle.value)
            elif buttonname.value == 'cyclo6':
                if strictele1.value[0] == 'X':
                    atoms_b.sub(buttonname.value, i.value, state.value, place.value,\
                                shuffle=shuffle.value, angle=angle.value)
                elif strictadd2.value == False:
                    atoms_b.sub(buttonname.value, i.value, state.value, place.value,\
                                strict={strictnum1:strictele1},\
                                shuffle=shuffle.value, angle=angle.value)
                elif strictadd3.value == False:
                    atoms_b.sub(buttonname.value, i.value, state.value, place.value,\
                                strict={strictnum1:strictele1, strictnum2:strictele2},\
                                shuffle=shuffle.value, angle=angle.value)
                else:
                    atoms_b.sub(buttonname.value, i.value, state.value, place.value,\
                                strict={strictnum1:strictele1, strictnum2:strictele2,\
                                        strictnum3:strictele3},\
                                shuffle=shuffle.value, angle=angle.value)
            else:
                atoms_b.sub(buttonname.value, i.value, state.value, place.value,\
                            shuffle=shuffle.value, angle=angle.value)
        elif choose.value == 'Ringbinders':
            if buttonname.value == 'bind3':
                atoms_b.bind(buttonname.value, [i1.value, i2.value],\
                             state=[state1.value, state2.value], place=place.value,\
                             strict={strictnum1:strictele1}, side=side.value,\
                             shuffle=shuffle.value, less5=less5.value, stable=stable.value)
            elif buttonname.value in ['SmallChanges', 'bindconj5', 'bindconj6']:
                if strictele1.value[0] == 'X':
                    atoms_b.bind(buttonname.value, [i1.value, i2.value],\
                                 state=[state1.value, state2.value], place=place.value,\
                                 side=side.value, shuffle=shuffle.value,\
                                 less5=less5.value, stable=stable.value)
                elif strictadd2.value == False:
                    atoms_b.bind(buttonname.value, [i1.value, i2.value],\
                                 state=[state1.value, state2.value], place=place.value,\
                                 strict={strictnum1:strictele1}, side=side.value,\
                                 shuffle=shuffle.value, less5=less5.value, stable=stable.value)
                else:
                    atoms_b.bind(buttonname.value, [i1.value, i2.value],\
                                 state=[state1.value, state2.value], place=place.value,\
                                 strict={strictnum1:strictele1, strictnum2:strictele2},\
                                 side=side.value, shuffle=shuffle.value,\
                                 less5=less5.value, stable=stable.value)
            else:
                atoms_b.bind(buttonname.value, [i1.value, i2.value],\
                             state=[state1.value, state2.value], place=place.value,\
                             side=side.value, shuffle=shuffle.value,\
                             less5=less5.value, stable=stable.value)
        elif choose.value == 'SmallChanges':
            atoms_b.C(buttonname.value, i.value, state=[state1.value, state2.value])
        atoms_b.xyzwrite('Mole_Untitled_beta')
        with output3:
            view3d('Mole_Untitled_beta', loc.value, width=400, height=500)
            display(widgets.HBox([savename, savefile]))
        file.value='Mole_Untitled'
        os.system('rm Mole_Untitled_beta.xyz')
        atoms = molecule('Mole_Untitled', loc.value)

    def settlechange(button):
        global atoms
        output3.clear_output()
        if choose.value == 'Substrates':
            if buttonname.value == 'cyclo3':
                if strictele1.value[0] == 'X':
                    atoms.sub(buttonname.value, i.value, state.value, place.value,\
                              shuffle=shuffle.value, angle=angle.value)
                else:
                    atoms.sub(buttonname.value, i.value, state.value, place.value,\
                              strict={strictnum1:strictele1},\
                              shuffle=shuffle.value, angle=angle.value)
            elif buttonname.value == 'cyclo5':
                if strictele1.value[0] == 'X':
                    atoms.sub(buttonname.value, i.value, state.value, place.value,\
                              shuffle=shuffle.value, angle=angle.value)
                elif strictadd2.value == False:
                    atoms.sub(buttonname.value, i.value, state.value, place.value,\
                              strict={strictnum1:strictele1},\
                              shuffle=shuffle.value, angle=angle.value)
                else:
                    atoms.sub(buttonname.value, i.value, state.value, place.value,\
                              strict={strictnum1:strictele1, strictnum2:strictele2},\
                              shuffle=shuffle.value, angle=angle.value)
            elif buttonname.value == 'cyclo6':
                if strictele1.value[0] == 'X':
                    atoms.sub(buttonname.value, i.value, state.value, place.value,\
                              shuffle=shuffle.value, angle=angle.value)
                elif strictadd2.value == False:
                    atoms.sub(buttonname.value, i.value, state.value, place.value,\
                              strict={strictnum1:strictele1},\
                              shuffle=shuffle.value, angle=angle.value)
                elif strictadd3.value == False:
                    atoms.sub(buttonname.value, i.value, state.value, place.value,\
                              strict={strictnum1:strictele1, strictnum2:strictele2},\
                              shuffle=shuffle.value, angle=angle.value)
                else:
                    atoms.sub(buttonname.value, i.value, state.value, place.value,\
                              strict={strictnum1:strictele1, strictnum2:strictele2,\
                                      strictnum3:strictele3},\
                              shuffle=shuffle.value, angle=angle.value)
            else:
                atoms.sub(buttonname.value, i.value, state.value,\
                          place.value, shuffle=shuffle.value, angle=angle.value)
        elif choose.value == 'Ringbinders':
            if buttonname.value == 'bind3':
                atoms.bind(buttonname.value, [i1.value, i2.value],\
                           state=[state1.value, state2.value], place=place.value,\
                           strict={strictnum1:strictele1}, side=side.value,\
                           shuffle=shuffle.value, less5=less5.value, stable=stable.value)
            elif buttonname.value in ['SmallChanges', 'bindconj5', 'bindconj6']:
                if strictele1.value[0] == 'X':
                    atoms.bind(buttonname.value, [i1.value, i2.value],\
                               state=[state1.value, state2.value], place=place.value,\
                               side=side.value, shuffle=shuffle.value, less5=less5.value,\
                               stable=stable.value)
                elif strictadd2.value == False:
                    atoms.bind(buttonname.value, [i1.value, i2.value],\
                               state=[state1.value, state2.value], place=place.value,\
                               strict={strictnum1:strictele1}, side=side.value,\
                               shuffle=shuffle.value, less5=less5.value, stable=stable.value)
                else:
                    atoms.bind(buttonname.value, [i1.value, i2.value],\
                               state=[state1.value, state2.value], place=place.value,\
                               strict={strictnum1:strictele1, strictnum2:strictele2},\
                               side=side.value, shuffle=shuffle.value,\
                               less5=less5.value, stable=stable.value)
            else:
                atoms.bind(buttonname.value, [i1.value, i2.value],\
                           state=[state1.value, state2.value], place=place.value, side=side.value,\
                           shuffle=shuffle.value, less5=less5.value, stable=stable.value)
        elif choose.value == 'SmallChanges':
            atoms.C(buttonname.value, i.value, state=[state1.value, state2.value])
        atoms.xyzwrite('Mole_Untitled')
        atoms = molecule('Mole_Untitled', loc.value)
        file.value='Mole_Untitled'
        with output3:
            view3d('Mole_Untitled', loc.value, width=400, height=500)
            display(widgets.HBox([savename, savefile]))
    
    def strictout2(strictadd2):
        strictoutput2.clear_output()
        if strictadd2 == True:
            with strictoutput2:
                if buttonname.value in ('cyclo5', 'bind5', 'bind6', 'bindconj5', 'bindconj6'):
                    display(widgets.VBox([strictnum2, strictele2]))
                else:
                    display(widgets.VBox([strictnum2, widgets.HBox([strictele2, strictadd3])]))
    
    def strictout3(strictadd3):
        strictoutput3.clear_output()
        if strictadd3 == True:
            with strictoutput3:
                display(widgets.VBox([strictnum3, strictele3]))
    
    def angleavailout(angleavail):
        if angleavail == True:
            angle.disabled = True
        else:
            angle.disabled = False

    def chooseshow(i, i1, i2, choose):
        output3.clear_output()
        with output3:
            try:
                if choose == 'Substrates':
                    view3dchoose(file.value, loc.value, choose=[i], width=400, height=500)
                elif choose == 'Ringbinders':
                    view3dchoose(file.value, loc.value, choose=[i1, i2], width=400, height=500)
                elif choose == 'SmallChanges':
                    view3dchoose(file.value, loc.value, choose=[i], width=400, height=500)
                display(widgets.HBox([savename, savefile]))
            except:
                pass
    
    def default(strictele1):
        if strictele1 == 'X(Default)':
            strictnum1.value=1
            strictadd2.disabled = True
        else:
            strictadd2.disabled = False
    
    output0 = widgets.Output()
    output = widgets.Output()
    output2 = widgets.Output()
    output3 = widgets.Output()
    strictoutput1 = widgets.Output()
    strictoutput2 = widgets.Output()
    strictoutput3 = widgets.Output()
    angleoutput = widgets.Output()
    placeoutput = widgets.Output()
    shuffleoutput = widgets.Output()
    
    options = []
    loc = widgets.Text(description='Location:', value='./')
    for path in os.listdir(loc.value):
        if os.path.isfile(os.path.join(loc.value, path)):
            a = re.search('.xyz', path)
            if a:
                options.append(path[:-4])
    file = widgets.Combobox(placeholder = 'Input the file name', options=options,\
             description='xyz File:')
    
    openfile = widgets.Button(description='Set')
    openfile.on_click(openf)
    
    smart = widgets.Text(description='SMART:',\
                         placeholder = 'Input SMART structure')
    smartbutton = widgets.Button(description='Set')
    smartbutton.on_click(smartset)
    
    i = widgets.BoundedIntText(description='Atom Number:',\
                               min=0, style={'description_width': 'initial'})
    state = widgets.Dropdown(description='Environment:',\
                             options=[('1', 1), ('2', 2), ('4', 4), ('6', 6), ('7', 7),\
                                      ('8', 8), ('16', 16), ('82', 82), ('162', 162), ('164', 164)],\
                             value=1, style={'description_width': 'initial'})
    place = widgets.BoundedIntText(description='Connect Place:',\
                                   min=0, value=0, style={'description_width': 'initial'})
    strictnum1 = widgets.BoundedIntText(description='Hetero Place 1:',\
                                        min=1, value=1, style={'description_width': 'initial'})
    strictele1 = widgets.Dropdown(description='Hetero Elements 1:',\
                                  options=['N', 'O', 'S', 'X(Default)'], value='X(Default)',\
                                  style={'description_width': 'initial'})
    #strictdefault = widgets.ToggleButton(description='default', value=True)
    strictnum2 = widgets.BoundedIntText(description='Hetero Place 2:',\
                                        min=3, value=3, style={'description_width': 'initial'})
    strictele2 = widgets.Dropdown(description='Hetero Elements 2:',\
                                  options=['N', 'O', 'S'], style={'description_width': 'initial'})
    strictadd2 = widgets.ToggleButton(description='Add', value=False)
    strictnum3 = widgets.BoundedIntText(description='Hetero Place 3:',\
                                        min=5, value=5, style={'description_width': 'initial'})
    strictele3 = widgets.Dropdown(description='Hetero Elements 3:',\
                                  options=['N', 'O', 'S'], style={'description_width': 'initial'})
    strictadd3 = widgets.ToggleButton(description='Add', value=False)
    i1 = widgets.BoundedIntText(description='Atom Number 1:',\
                                min=0, style={'description_width': 'initial'})
    state1 = widgets.Dropdown(description='Environment:',\
                              options=[('1', 1), ('2', 2), ('6', 6)],\
                              value=1, style={'description_width': 'initial'})
    i2 = widgets.BoundedIntText(description='Atom Number 2:',\
                                min=0, style={'description_width': 'initial'})
    state2 = widgets.Dropdown(description='Environment:',\
                              options=[('1', 1), ('2', 2), ('6', 6)], value=1,\
                              style={'description_width': 'initial'})
    shuffle = widgets.BoundedIntText(description='Shuffle:', min=0, max=3,\
                                     value=0, style={'description_width': 'initial'})
    angle = widgets.BoundedFloatText(min=0, max=360, step = 0.1,\
                                     description='Angle:', style={'description_width': 'initial'})
    angleavail = widgets.ToggleButton(value=False, description='Default',\
                                      style={'description_width': 'initial'})
    side = widgets.BoundedIntText(description='Conformer:', min=0, max=3,\
                                  value=0, style={'description_width': 'initial'})
    shuffle = widgets.BoundedIntText(description='Shuffle:', min=0, value=0,\
                                     style={'description_width': 'initial'})
    less5 = widgets.Checkbox(description='Connect with small rings (member<=5)?',\
                             style={'description_width': 'initial'})
    stable = widgets.Checkbox(description='Most Stable structure?',\
                              style={'description_width': 'initial'})
    preview = widgets.Button(description='View')
    preview.on_click(viewset)
    settle = widgets.Button(description='Set')
    settle.on_click(settlechange)
    savename = widgets.Text(description='Save Name:', value='Mole_Untitled')
    savefile = widgets.Button(description='Save')
    savefile.on_click(saveset)
    buttonname = widgets.Text()
    
    #file = widgets.Button(description='Set')
    
    strict2 = widgets.interactive_output(strictout2, {'strictadd2': strictadd2})
    strict3 = widgets.interactive_output(strictout3, {'strictadd3': strictadd3})
    angleout = widgets.interactive_output(angleavailout,\
                                          {'angleavail': angleavail})
    smartout = widgets.interactive_output(smartshow, {'smart': smart})
    
    substrates = ['F', 'Cl', 'Br', 'I', 'OH', 'OMe', 'SH', 'SMe', 'NH2', 'NMe2',\
                  'Me', 'Et', 'n-Pr', 'i-Pr', 'n-Bu', 't-Bu', 'CF3', 'CCl3', 'CBr3',\
                  'CI3', 'NO2', 'C2H3', 'C=NH', 'CHO', 'COOH', 'COMe', 'CN', 'Ph',\
                  'pyrrole', 'Py', 'cyclo3', 'cyclo5', 'cyclo6', 'SOMe', 'SO2Me',\
                  'SO3H', 'Bpin', 'bicyclo[2.2.1]', 'bicyclo[2.2.2]']
    buttons1 = []
    for x in substrates:
        button = widgets.Button(layout = widgets.Layout(height='30px',\
                                                      min_width='120px'), description=x)
        buttons1.append(button)
        button.on_click(outputmol1)
    Substrates = widgets.HBox(
        children = [buttons1[x] for x in range(len(substrates))])
    
    binds = ['bind2', 'bind3', 'bind5', 'bind6', 'bindconj5', 'bindconj6']
    buttons2 = []
    for x in binds:
        button = widgets.Button(layout = widgets.Layout(height='30px',\
                                                        min_width='120px'), description=x)
        buttons2.append(button)
        button.on_click(outputmol2)
    Ringbinders = widgets.HBox(children = [buttons2[x] for x in range(len(binds))])
    
    Cs = ['C=O', 'C=NH', 'O', 'S', 'N']
    buttons3 = []
    for x in Cs:
        button = widgets.Button(layout = widgets.Layout(height='30px',\
                                                        min_width='120px'), description=x)
        buttons3.append(button)
        button.on_click(outputmol3)
    SmallChanges = widgets.HBox(children = [buttons3[x] for x in range(len(Cs))])
        
    stack = widgets.Stack([Substrates, Ringbinders, SmallChanges])
    
    choose=widgets.Dropdown(options=['Substrates', 'Ringbinders', 'SmallChanges'])
    widgets.jslink((choose, 'index'), (stack, 'selected_index'))
    
    changecomboout = widgets.interactive_output(changecombo, {'loc': loc})
    chooseout = widgets.interactive_output(chooseshow, {'i': i, 'i1': i1,\
                                                        'i2': i2, 'choose': choose})
    defaultout = widgets.interactive_output(default, {'strictele1': strictele1})
    
    box_layout = widgets.Layout(overflow='scroll hidden',\
                                border='2px solid black', width='65%', height='550px', display='flex')
    box_layout2 = widgets.Layout(overflow='scroll hidden',\
                                 border='2px solid black', width='35%', height='550px', display='flex')
    output3.layout = box_layout2
    clean = widgets.Button(description='Clean Data')
    clean.on_click(clear)
    
    with output0:
        display(widgets.VBox([loc, widgets.HBox([file, openfile]),\
                              widgets.Label('OR', style = dict(font_weight='bold',\
                                                               text_decoration='underline')),\
                              widgets.HBox([smart, smartbutton])]))
    
    allchoose = widgets.HBox([widgets.Label('Choose your step: '), choose, clean])
    display(widgets.HBox([widgets.VBox([output0, output],\
                                       layout=box_layout), output3]))


import sys
import os
import re
import ipywidgets as widgets
from .XyzToInp import XyzToInp
from .orca import orca
from .multiorca import multiorca
from .ssorca import ssorca
from ase.io import read
from ase import Atoms
from .view3dchoose import view3dchoose
from IPython.display import display

mydir = os.path.dirname( __file__ )
statusdir = os.path.join(mydir, '..', 'status')
sys.path.append(statusdir)
from status import status

mydir = os.path.dirname( __file__ )
viewdir = os.path.join(mydir, '..', 'view3d')
sys.path.append(viewdir)
from view3d import view3d

def gui():
    """
        The method to use GUI to set ORCA input files and calculate the file by ORCA get in handy!
        You will know how to do as long as you get into the GUI platform.

        TIPS: 1. Try to directly input the figure of external force instead of scrolling it, or you will see a lot of input files.
              2. If you specialized in calcuating mechanical properties, orcaset.ssgui is a better option.
              3. Take caution when you set Scanning or External Force site. Don't set number out of range.
              4. To see the status of your optimisation, you can set another window and use status.figuretraj .
              5. The visualisation will also risk the flexibility of the task, to make you programme more flexible, please directly use the code.
    """
    output = widgets.Output()
    output2 = widgets.Output()
    scanoutput = widgets.Output()
    forceoutput = widgets.Output()
    eoutput = widgets.Output()
    geooutput = widgets.Output()
    mdoutput = widgets.Output()
    global geore
        
    def scanshow(scanox):
        aim1.value = 0
        aim2.value = 0
        scanoutput.clear_output()
        opt.disabled = False
        ts.disabled = False
        freq.disabled = False
        forceox.disabled = False
        if scanox == True:
            opt.disabled = True
            ts.disabled = True
            freq.disabled = True
            forceox.value = False
            forceox.disabled = True
            with scanoutput:
                display(widgets.VBox([widgets.HBox([aim1, aim2]), stretch, scanstep]))

    def forceshow(forceox):
        aim1.value = 0
        aim2.value = 0
        forceoutput.clear_output()
        ts.disabled = False
        scanox.disabled = False
        if forceox == True:
            ts.disabled = True
            scanox.value = False
            scanox.disabled = True
            opt.value = True
            with forceoutput:
                display(widgets.VBox([widgets.HBox([aim1, aim2]), strain]))

    def eshow(eox):
        eoutput.clear_output()
        if eox == True:
            with eoutput:
                if scanox.value == True:
                    status(file.value+'_scan', loc.value).status()
                elif forceox.value == True:
                    status(file.value+'_{0:.3f}'.format(strain.value), loc.value).status()
                else:
                    status(file.value, loc.value).status()

    def geoshow(geoox):
        geooutput.clear_output()
        if geoox == True:
            with geooutput:
                if scanox.value == True:
                    geore = status(file.value+'_scan', loc.value)
                    savetotal = geore.steps()
                    savenumchoose.max = savetotal
                    def save3Dshow(savenumchoose):
                        savenum = savenumchoose
                        geore.figure(num=savenumchoose)
                    savenumchooseout = widgets.interactive_output(save3Dshow,\
                                                                  {'savenumchoose': savenumchoose})
                    display(savenumchoose)
                    display(savenumchooseout)
                elif forceox.value == True:
                    geore = status(file.value+'_{0:.3f}'.format(strain.value),\
                                   loc.value)
                    savetotal = geore.steps()
                    savenumchoose.max = savetotal
                    def save3Dshow(savenumchoose):
                        savenum = savenumchoose
                        geore.figure(num=savenumchoose)
                    savenumchooseout = widgets.interactive_output(save3Dshow,\
                                                                  {'savenumchoose': savenumchoose})
                    display(savenumchoose)
                    display(savenumchooseout)
                elif mdox.value == True:
                    geore = status(file.value+'_{0:.3f}'.format(strain.value),\
                                   loc.value)
                    savetotal = geore.steps(MD=True)
                    savenumchoose.max = savetotal
                    def save3Dshow(savenumchoose):
                        savenum = savenumchoose
                        geore.figure(num=savenumchoose)
                    savenumchooseout = widgets.interactive_output(save3Dshow,\
                                                                  {'savenumchoose': savenumchoose})
                    display(savenumchoose)
                    display(savenumchooseout)
                else:
                    geore = status(file.value, loc.value)
                    savetotal = geore.steps()
                    savenumchoose.max = savetotal
                    def save3Dshow(savenumchoose):
                        savenum = savenumchoose
                        geore.figure(num=savenumchoose)
                    savenumchooseout = widgets.interactive_output(save3Dshow,\
                                                                  {'savenumchoose': savenumchoose})
                    display(savenumchoose)
                    display(savenumchooseout)
    
    def mdshow(mdox):
        mdoutput.clear_output()
        if mdox == True:
            ts.disabled = True
            scanox.disabled = True
            opt.disabled = True
            freq.disabled = True
            forceox.disabled = True
            with mdoutput:
                display(widgets.VBox([widgets.HBox([tstep, initvel]),\
                                      widgets.HBox([tsNHC, timecon]), dumpcon, runstep]))
        else:
            ts.disabled = False
            scanox.disabled = False
            opt.disabled = False
            freq.disabled = False
            forceox.disabled = False
    
    def changecombo(loc):
        options2 = []
        for path in os.listdir(loc):
            if os.path.isfile(os.path.join(loc, path)):
                a = re.search('.out', path)
                if a:
                    options2.append(path[:-4])
        options2.sort()
        file.options = options2
    
    def guiconvert(file, loc, method, bs, opt, freq, ts, scanox,\
                   forceox, aim1, aim2, scanstep, stretch, strain,\
                   room, roomox, corenum, coreox, e, state, maxiter,\
                   mdox, tstep, initvel, tsNHC, timecon, dumpcon, runstep):
        if scanox == True or forceox == True:
            atomsx = read(loc+file+'.xyz')
            num = len(atomsx)
            if aim1 >= num:
                aim1 = num - 1
            if aim2 >= num:
                aim2 = num - 1
        try:
            if roomox == True and coreox == True:
                if scanox == True:
                    XyzToInp(file, inpname=file+'_scan', fileloc=loc, method=method,\
                             basis_set=bs, opt=opt, scan=True, aim = [aim1, aim2],\
                             stretch=stretch, scanstep=scanstep, maxiter=maxiter,\
                             electron=e, state=state, MD=mdox, md_timestep=tstep,\
                             initvel=initvel, tsNHC=tsNHC, timecon=timecon,\
                             dumpcon=dumpcon, runstep=runstep)
                elif forceox == True:
                    XyzToInp(file, inpname = '{0}_{1:.3f}'.format(file, strain),\
                             fileloc=loc, method=method, basis_set=bs, opt=opt, freq=freq,\
                             external_force=True, aim = [aim1, aim2], strain=strain,\
                             maxiter=maxiter, electron=e, state=state, MD=mdox, md_timestep=tstep,\
                             initvel=initvel, tsNHC=tsNHC, timecon=timecon,\
                             dumpcon=dumpcon, runstep=runstep)
                else:
                    XyzToInp(file, fileloc=loc, method=method, basis_set=bs, opt=opt,\
                             freq=freq, ts = ts, electron=e, state=state, MD=mdox,\
                             md_timestep=tstep, initvel=initvel, tsNHC=tsNHC, timecon=timecon,\
                             dumpcon=dumpcon, runstep=runstep)
            elif roomox == False and coreox == False:
                if scanox == True:
                    XyzToInp(file, inpname = file+'_scan', fileloc=loc, method=method,\
                             basis_set=bs, opt=opt, scan=True, aim = [aim1, aim2],\
                             stretch=stretch, scanstep=scanstep, maxiter=maxiter,\
                             maxcore = room*1024, corenum=corenum, electron=e, state=state,\
                             MD=mdox, md_timestep=tstep, initvel=initvel, tsNHC=tsNHC,\
                             timecon=timecon, dumpcon=dumpcon, runstep=runstep)
                elif forceox == True:
                    XyzToInp(file, inpname = '{0}_{1:.3f}'.format(file, strain), fileloc=loc,\
                             method=method, basis_set=bs, opt=opt, freq=freq, external_force=True,\
                             aim = [aim1, aim2], strain=strain, maxiter=maxiter,\
                             maxcore = room*1024, corenum=corenum, electron=e, state=state,\
                             MD=mdox, md_timestep=tstep, initvel=initvel, tsNHC=tsNHC,\
                             timecon=timecon, dumpcon=dumpcon, runstep=runstep)
                else:
                    XyzToInp(file, fileloc=loc, method=method, basis_set=bs, opt=opt, freq=freq,\
                             ts=ts, maxiter=maxiter, maxcore = room*1024, corenum=corenum,\
                             electron=e, state=state, MD=mdox, md_timestep=tstep, initvel=initvel,\
                             tsNHC=tsNHC, timecon=timecon, dumpcon=dumpcon, runstep=runstep)
            elif roomox == True and coreox == False:
                if scanox == True:
                    XyzToInp(file, inpname = file+'_scan', fileloc=loc, method=method,\
                             basis_set=bs, opt=opt, scan=True, aim = [aim1, aim2],\
                             stretch=stretch, scanstep=scanstep, maxiter=maxiter, corenum=corenum,\
                             electron=e, state=state, MD=mdox, md_timestep=tstep, initvel=initvel,\
                             tsNHC=tsNHC, timecon=timecon, dumpcon=dumpcon, runstep=runstep)
                elif forceox == True:
                    XyzToInp(file, inpname = '{0}_{1:.3f}'.format(file, strain), fileloc=loc,\
                             method=method, basis_set=bs, opt=opt, freq=freq, external_force=True,\
                             aim = [aim1, aim2], strain=strain, maxiter=maxiter, corenum=corenum,\
                             electron=e, state=state, MD=mdox, md_timestep=tstep, initvel=initvel,\
                             tsNHC=tsNHC, timecon=timecon, dumpcon=dumpcon, runstep=runstep)
                else:
                    XyzToInp(file, inpname = file+'_scan', fileloc=loc, method=method,\
                             basis_set=bs, opt=opt, freq=freq, ts=ts, maxiter=maxiter,\
                             corenum=corenum, electron=e, state=state, MD=mdox, md_timestep=tstep,\
                             initvel=initvel, tsNHC=tsNHC, timecon=timecon,\
                             dumpcon=dumpcon, runstep=runstep)
            elif roomox == False and coreox == True:
                if scanox == True:
                    XyzToInp(file, inpname = file+'_scan', fileloc=loc, method=method,\
                             basis_set=bs, opt=opt, scan=True, aim = [aim1, aim2],\
                             stretch=stretch, scanstep=scanstep, maxiter=maxiter,\
                             maxcore = room*1024, electron=e, state=state, MD=mdox,\
                             md_timestep=tstep, initvel=initvel, tsNHC=tsNHC, timecon=timecon,\
                             dumpcon=dumpcon, runstep=runstep)
                elif forceox == True:
                    XyzToInp(file, inpname = '{0}_{1:.3f}'.format(file, strain), fileloc=loc,\
                             method=method, basis_set=bs, opt=opt, freq=freq, external_force=True,\
                             aim = [aim1, aim2], strain=strain, maxiter=maxiter,\
                             maxcore = room*1024, electron=e, state=state, MD=mdox,\
                             md_timestep=tstep, initvel=initvel, tsNHC=tsNHC,\
                             timecon=timecon, dumpcon=dumpcon, runstep=runstep)
                else:
                    XyzToInp(file, fileloc=loc, method=method, basis_set=bs, opt=opt,\
                             freq=freq, ts=ts, maxiter=maxiter, maxcore = room*1024,\
                             electron=e, state=state, MD=mdox, md_timestep=tstep, initvel=initvel,\
                             tsNHC=tsNHC, timecon=timecon, dumpcon=dumpcon, runstep=runstep)

        except:
            print('Setting Error.')
            return None
        if scanox == True:
            print(file+'_scan'+'.inp\n')
            f = open(loc+file+'_scan'+'.inp', 'r')
        elif forceox == True:
            print('{0}_{1:.3f}.inp\n'.format(file, strain))
            f = open(loc+'{0}_{1:.3f}.inp'.format(file, strain), 'r')
        else:
            print(file+'.inp\n')
            f = open(loc+file+'.inp', 'r')
        for line in f:
            print(line[:-1])
        f.close()
        if forceox == True:
            os.system('rm {2}/{0}_{1:.3f}.inp'.format(file, strain, loc))

    def screen3d(file, loc, aim1, aim2, scanox, forceox):
        aim = []
        try:
            if scanox == True or forceox == True:
                atomsx = read(loc+file+'.xyz')
                num = len(atomsx)
                if aim1 >= num or aim2 >= num:
                    print('Setting Error. The maximum number of this system is {0}.\n\
                        We will automatically change\
                        the number into the maximum number.'.format(num-1))
                if aim1 >= num:
                    aim1 = num - 1
                if aim2 >= num:
                    aim2 = num - 1
                aim = [aim1, aim2]
                print('Atom1: {0}      Atom2: {1}'.format(aim1, aim2))
                view3dchoose(file, loc, choose=aim, width=400, height=400)
            else:
                view3d(file, loc, width=400, height=500)
        except:
            print('Setting Error.')

    def createinp(button):
        output.clear_output()
        output2.clear_output()
        with output:
            out = widgets.interactive_output(guiconvert, {'file': file, 'loc': loc,\
                                                          'method': method, 'bs': bs, 'opt': opt, 'freq': freq, \
                                                          'ts': ts, 'scanox': scanox, 'forceox': forceox, 'aim1': aim1,\
                                                          'aim2': aim2,'stretch': stretch, 'scanstep': scanstep,\
                                                          'strain': strain, 'maxiter': maxiter, 'room': room, \
                                                          'roomox': roomox, 'corenum': corenum, 'coreox': coreox,\
                                                          'e': e, 'state': state, 'mdox': mdox, 'tstep': tstep,\
                                                          'initvel': initvel, 'tsNHC': tsNHC, 'timecon': timecon,\
                                                          'dumpcon': dumpcon, 'runstep': runstep})
            startorca = widgets.Button(description = 'Start ORCA Calculation',\
                                       layout = widgets.Layout(width='99%'),\
                                       style = dict(font_weight='bold', text_decoration='underline'))
            startorca.on_click(intro_status3d)
            display(widgets.VBox([out, widgets.HBox([widgets.Label('ORCA Location:'),\
                                                     orcaloc]), startorca]))
        with output2:
            out2 = widgets.interactive_output(screen3d, {'file': file, 'loc': loc,\
                                                         'aim1': aim1, 'aim2': aim2, 'scanox': scanox, 'forceox': forceox}) 
            display(out2)

    def intro_status3d(button):
        if scanox.value == True or forceox.value == True:
            atomsx = read(loc.value+file.value+'.xyz')
            num = len(atomsx)
            if aim1.value >= num:
                aim1.value = num - 1
            if aim2.value >= num:
                aim2.value = num - 1
        with output:
            button.description = 'Processing...'
        if scanox.value == True:
            orca(file.value+'_scan', orcaloc=orcaloc.value)
        elif forceox.value == True:
            if roomox.value == True and coreox.value == True:
                XyzToInp(file.value,\
                         inpname = '{0}_{1:.3f}'.format(file.value, strain.value),\
                         fileloc=loc.value, method=method.value, basis_set=bs.value,\
                         opt=opt.value, freq=freq.value, external_force=True,\
                         aim = [aim1.value, aim2.value], strain=strain.value,\
                         maxiter=maxiter.value, electron=e.value, state=state.value)
            elif roomox.value == False and coreox.value == False:
                XyzToInp(file.value,\
                         inpname = '{0}_{1:.3f}'.format(file.value, strain.value),\
                         fileloc=loc.value, method=method.value, basis_set=bs.value,\
                         opt=opt.value, freq=freq.value, external_force=True,\
                         aim = [aim1.value, aim2.value], strain=strain.value,\
                         maxiter=maxiter.value, maxcore=room.value*1024,\
                         corenum=corenum.value, electron=e.value, state=state.value)
            else:
                XyzToInp(file.value,\
                         inpname = '{0}_{1:.3f}'.format(file.value, strain.value),\
                         fileloc=loc.value, method=method.value, basis_set=bs.value,\
                         opt=opt.value, freq=freq.value, external_force=True,\
                         aim = [aim1.value, aim2.value], strain=strain.value,\
                         maxiter=maxiter.value, maxcore=room.value*1024,\
                         electron=e.value, state=state.value)
            orca('{0}_{1:.3f}'.format(file.value, strain.value), orcaloc=orcaloc.value)
        else:
            orca(file.value, orcaloc=orcaloc.value)
        with output2:
            if scanox.value == True:
                if status(file.value+'_scan', loc.value).status(figureonly=True,\
                                                                statusonly=True) == 2:
                    display(widgets.HBox([widgets.Valid(description = file.value+'_scan',\
                                                        value=True), widgets.Label('  Calculation completed!')]))
                elif status(file.value+'_scan', loc.value).status(figureonly=True,\
                                                                  statusonly=True) == 4:
                    display(widgets.Valid(description = file.value+'_scan', value=False,\
                                          readout='Calculation aborted!'))
            elif forceox.value == True:
                if status('{0}_{1:.3f}'.format(file.value, strain.value),\
                          loc.value).status(figureonly=True, statusonly=True) == 2:
                    display(widgets.HBox([
                        widgets.Valid(description = '{0}_{1:.3f}'.format(file.value, strain.value),
                                      value=True), widgets.Label('  Calculation completed!')
                    ]))
                elif status('{0}_{1:.3f}'.format(file.value, strain.value),\
                            loc.value).status(figureonly=True, statusonly=True) == 4:
                    display(widgets.Valid(description = '{0}_{1:.3f}'.format(file.value,\
                                                                             strain.value), value=False, readout='Calculation aborted!'))
            else:
                if status(file.value, loc.value).status(figureonly=True,\
                                                        statusonly=True) == 2:
                    display(widgets.HBox([widgets.Valid(description = file.value,\
                                                        value=True), widgets.Label('  Calculation completed!')]))
                elif status(file.value, loc.value).status(figureonly=True,\
                                                          statusonly=True) == 4:
                    display(widgets.Valid(description = file.value,\
                                          value=False, readout='Calculation aborted!'))
            display(introstatus)
        output.clear_output()
        
    def status3d(button):
        output2.clear_output()
        with output2:
            display(widgets.VBox([eox, eoutput, geoox, geooutput,\
                                  widgets.HBox([widgets.Label('Save Name:'), savename, save])]))
    
    def saved(button):
        status(file.value, loc.value).figure(num=savenumchoose.value,\
                                             save=savename.value)

    options = []
    loc = widgets.Text(description = 'Location:', value='./')
    for path in os.listdir(loc.value):
        if os.path.isfile(os.path.join(loc.value, path)):
            a = re.search('.xyz', path)
            if a:
                options.append(path[:-4])
    file = widgets.Combobox(placeholder='Input the file name', options=options,\
                            description = 'xyz File:')
    method = widgets.Text(description = 'Method:')
    bs = widgets.Text(description = 'Basis Set:')
    opt = widgets.ToggleButton(value=False, description = 'Optimisation',\
                               layout = widgets.Layout(width='33.3%'))
    ts = widgets.ToggleButton(value=False, description = 'TS',\
                              layout = widgets.Layout(width='33.3%'))
    freq = widgets.ToggleButton(value=False, description = 'Frequency',\
                                layout = widgets.Layout(width='33.3%'))
    room = widgets.IntText(description = 'Space(GB):', value= 4, min=1,\
                           layout = widgets.Layout(width='50%'))
    roomox = widgets.ToggleButton(value=False, description = 'Default Space')
    corenum=widgets.IntText(description = 'Core:', value=4, min=1,\
                            layout = widgets.Layout(width='50%'))
    coreox = widgets.ToggleButton(value=False, description = 'Default CoreNumber')
    e = widgets.IntText(description = 'Charge:', value=0,\
                        layout = widgets.Layout(width='50%'))
    state = widgets.IntText(description = 'State:', value=1,\
                            layout = widgets.Layout(width='47.5%'))
    scanox = widgets.ToggleButton(value=False, description = 'Scan Settings',\
                                  style = dict(font_weight = 'bold'), layout = widgets.Layout(width='98.5%'))
    forceox = widgets.ToggleButton(value=False,\
                                   description = 'Constant External Force Settings',\
                                   style = dict(font_weight = 'bold'), layout = widgets.Layout(width='98.5%'))
    mdox = widgets.ToggleButton(value=False, description = 'Molecular Dynamics',\
                                style = dict(font_weight = 'bold'), layout = widgets.Layout(width='98.5%'))
    aim1 = widgets.BoundedIntText(description = 'Atom 1:', value=0,\
                                  min=0, max=300, layout = widgets.Layout(width='50%'))
    aim2 = widgets.BoundedIntText(description = 'Atom 2:', value=0,\
                                  min=0, max=300, layout = widgets.Layout(width='47.5%'))
    strain=widgets.BoundedFloatText(description = 'Force(nN):',\
                                    value=0.000, min=0.000, step=0.100)
    stretch=widgets.FloatText(description = 'Distance(Å):',\
                              value=1.000, step=0.100)
    scanstep=widgets.BoundedIntText(description = 'Steps:', value=10, min=0)
    scanout = widgets.interactive_output(scanshow, {'scanox': scanox})
    forceout = widgets.interactive_output(forceshow, {'forceox': forceox})
    eox = widgets.ToggleButton(value=False, description = 'Energy Status',\
                               style = dict(font_weight = 'bold'), layout = widgets.Layout(width='98.5%'))
    geoox = widgets.ToggleButton(value=False, description = 'Structure Status',\
                                 style = dict(font_weight = 'bold'), layout = widgets.Layout(width='98.5%'))
    maxiter=widgets.IntText(description = 'MaxIteration:', value=200,\
                            layout = widgets.Layout(width='98.5%'))
    tstep=widgets.BoundedFloatText(description = 'Timestep:', min=0, value=1.0,\
                                   step=0.100, layout = widgets.Layout(width='47.5%'))
    initvel = widgets.BoundedFloatText(description = 'Initial T (K):', min=0,\
                                       max=2000, value=300, step=0.100, layout = widgets.Layout(width='47.5%'))
    tsNHC = widgets.BoundedFloatText(description = 'NHC T (K):', min=0, max=2000,\
                                     value=300, step=1.000, layout = widgets.Layout(width='47.5%'))
    timecon = widgets.BoundedFloatText(description = 'Time Gap:', min=0,\
                                       value=10.0, step=0.100, layout = widgets.Layout(width='47.5%'))
    dumpcon = widgets.BoundedIntText(description = 'Dump Gap:', min=1,\
                                     value=10, step=1, layout = widgets.Layout(width='98.5%'))
    runstep=widgets.BoundedIntText(description = 'Run Steps:', min=1,\
                                   value=100, max=1000000, step=1, layout = widgets.Layout(width='98.5%'))
    savename = widgets.Text()
    save = widgets.Button(description = 'Save')
    savenumchoose = widgets.IntSlider(min=0, step=1)
    
    changecomboout = widgets.interactive_output(changecombo, {'loc': loc})
    eout = widgets.interactive_output(eshow, {'eox': eox})
    geoout = widgets.interactive_output(geoshow, {'geoox': geoox})
    mdout = widgets.interactive_output(mdshow, {'mdox': mdox})
    orcaloc = widgets.Text(value='./')
    
    title1 = widgets.Label('File Details', style = dict(font_weight = 'bold'))
    title2 = widgets.Label('Calculation Methods',\
                           style = dict(font_weight = 'bold'))
    
    box_layout1 = widgets.Layout(display = 'flex', flew_flow = 'column',\
                                 overflow = 'scroll hidden', align_items = 'stretch',\
                                 border = 'solid', height = '650px', width='31%')
    box_layout2 = widgets.Layout(display = 'flex', flew_flow = 'column',\
                                 overflow = 'scroll hidden', align_items = 'stretch',\
                                 border = 'solid', height = '650px', width='33%')
    widgets.jslink((roomox, 'value'), (room, 'disabled'))
    widgets.jslink((coreox, 'value'), (corenum, 'disabled'))
    create = widgets.Button(description = 'Preview ORCA Input File',\
                            style = dict(font_weight = 'bold', text_decoration = 'underline'),\
                            layout = widgets.Layout(width='98.5%'))
    introstatus = widgets.Button(description = 'See Current Calculation Status',\
                                 style = dict(font_weight = 'bold', text_decoration = 'underline'),\
                                 layout = widgets.Layout(width='98.5%'))

    frame1 = widgets.VBox([title1, loc, file, title2,\
                           widgets.HBox([opt, ts, freq]), method, bs,\
                           widgets.HBox([corenum, coreox]), widgets.HBox([room, roomox]),\
                           widgets.HBox([e, state]), maxiter, scanox, scanoutput, forceox,\
                           forceoutput, mdox, mdoutput, create], layout = box_layout1)
    output.layout = box_layout2
    output2.layout = box_layout2
    create.on_click(createinp)
    save.on_click(saved)
    introstatus.on_click(status3d)

    display(widgets.HBox([frame1, output, output2]))